import os

options(
    setup=Bunch(
        name="uRSSus",
        packages=['urssus', 'urssus.ui', 'urssus.templates'],
        version="0.1.0",
        author="Roberto Alsina",
        author_email='ralsina@netmanagers.com.ar',  
        package_data = {'urssus.templates': ['*.tmpl']}, 
        entry_points = {'gui_scripts': ['urssus = urssus.urssus:main']}, 
        install_requires = ['SQLAlchemy==0.4.6', 'Elixir==0.5.2', 'processing'], 
        description = 'A multiplatform GUI news agregator.', 
        license = 'GPLv2', 
        keywords = 'atom rss pyqt', 
        url = 'http://urssus.googlecode.com', 
        long_description = '''
uRSSus is a multi-platform RSS/Atom news aggregtor, with a PyQt4 GUI. 
The code is hosted at http://urssus.googlecode.com and the license is GPL v2.0.

Hope you enjoy it, please use googlecode's issues_ page for bug reports, or mail me at
ralsina at netmanagers dot com dot ar.
        
.. _issues: http://code.google.com/p/urssus/issues/list
'''
    )
)

@task
@needs(['compile_resource','compile_ui','generate_setup', 'minilib', 'setuptools.command.sdist'])
def sdist():
  """Overrides sdist to make sure that our setup.py is generated."""
  pass
  
  
uidir=os.path.join('urssus', 'ui')

@task
def compile_ui():
  '''Compile the .ui files using pyuic4'''
  for f in os.listdir(uidir):
    if f.endswith('.ui'):
      print "Compiling ", f
      os.system ('pyuic4 %s -o %s'%(os.path.join(uidir, f), os.path.join(uidir, 'Ui_%s.py'%f[:-3])))
      
@task
def compile_resource():
  '''Compile the icons/images resource file'''
  os.system ('pyrcc4 %s -o %s'%(os.path.join('urssus','images', 'icons.qrc'), os.path.join(uidir, 'icons_rc.py')))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'urssus/ui/feed_properties.ui'
#
# Created: Sun Jul 27 21:58:01 2008
#      by: PyQt4 UI code generator 4.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(387,287)
        self.verticalLayout_2 = QtGui.QVBoxLayout(Dialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.tabWidget = QtGui.QTabWidget(Dialog)
        self.tabWidget.setObjectName("tabWidget")
        self.general = QtGui.QWidget()
        self.general.setGeometry(QtCore.QRect(0,0,365,200))
        self.general.setObjectName("general")
        self.verticalLayout = QtGui.QVBoxLayout(self.general)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtGui.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.url = QtGui.QLineEdit(self.general)
        self.url.setObjectName("url")
        self.gridLayout.addWidget(self.url,1,1,1,1)
        self.label_2 = QtGui.QLabel(self.general)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2,1,0,1,1)
        self.label = QtGui.QLabel(self.general)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label,0,0,1,1)
        self.name = QtGui.QLineEdit(self.general)
        self.name.setObjectName("name")
        self.gridLayout.addWidget(self.name,0,1,1,1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.notify = QtGui.QCheckBox(self.general)
        self.notify.setObjectName("notify")
        self.verticalLayout.addWidget(self.notify)
        self.customUpdate = QtGui.QCheckBox(self.general)
        self.customUpdate.setObjectName("customUpdate")
        self.verticalLayout.addWidget(self.customUpdate)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_3 = QtGui.QLabel(self.general)
        self.label_3.setEnabled(False)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout.addWidget(self.label_3)
        self.updatePeriod = QtGui.QSpinBox(self.general)
        self.updatePeriod.setEnabled(False)
        self.updatePeriod.setMinimum(1)
        self.updatePeriod.setMaximum(500)
        self.updatePeriod.setObjectName("updatePeriod")
        self.horizontalLayout.addWidget(self.updatePeriod)
        self.updateUnit = QtGui.QComboBox(self.general)
        self.updateUnit.setEnabled(False)
        self.updateUnit.setObjectName("updateUnit")
        self.horizontalLayout.addWidget(self.updateUnit)
        self.verticalLayout.addLayout(self.horizontalLayout)
        spacerItem = QtGui.QSpacerItem(20,40,QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.tabWidget.addTab(self.general,"")
        self.archive = QtGui.QWidget()
        self.archive.setGeometry(QtCore.QRect(0,0,365,200))
        self.archive.setObjectName("archive")
        self.verticalLayout_3 = QtGui.QVBoxLayout(self.archive)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.gridLayout_2 = QtGui.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.useDefault = QtGui.QRadioButton(self.archive)
        self.useDefault.setChecked(True)
        self.useDefault.setObjectName("useDefault")
        self.gridLayout_2.addWidget(self.useDefault,0,0,1,2)
        self.keepAll = QtGui.QRadioButton(self.archive)
        self.keepAll.setObjectName("keepAll")
        self.gridLayout_2.addWidget(self.keepAll,1,0,1,2)
        self.limitCount = QtGui.QRadioButton(self.archive)
        self.limitCount.setObjectName("limitCount")
        self.gridLayout_2.addWidget(self.limitCount,2,0,1,1)
        self.count = QtGui.QSpinBox(self.archive)
        self.count.setEnabled(False)
        self.count.setObjectName("count")
        self.gridLayout_2.addWidget(self.count,2,1,1,1)
        self.limitDays = QtGui.QRadioButton(self.archive)
        self.limitDays.setObjectName("limitDays")
        self.gridLayout_2.addWidget(self.limitDays,3,0,1,1)
        self.days = QtGui.QSpinBox(self.archive)
        self.days.setEnabled(False)
        self.days.setObjectName("days")
        self.gridLayout_2.addWidget(self.days,3,1,1,1)
        self.noArchive = QtGui.QRadioButton(self.archive)
        self.noArchive.setObjectName("noArchive")
        self.gridLayout_2.addWidget(self.noArchive,4,0,1,2)
        self.verticalLayout_3.addLayout(self.gridLayout_2)
        spacerItem1 = QtGui.QSpacerItem(20,40,QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem1)
        self.tabWidget.addTab(self.archive,"")
        self.advanced = QtGui.QWidget()
        self.advanced.setGeometry(QtCore.QRect(0,0,365,200))
        self.advanced.setObjectName("advanced")
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.advanced)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.loadFull = QtGui.QCheckBox(self.advanced)
        self.loadFull.setObjectName("loadFull")
        self.verticalLayout_4.addWidget(self.loadFull)
        self.markRead = QtGui.QCheckBox(self.advanced)
        self.markRead.setObjectName("markRead")
        self.verticalLayout_4.addWidget(self.markRead)
        spacerItem2 = QtGui.QSpacerItem(20,123,QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem2)
        self.tabWidget.addTab(self.advanced,"")
        self.verticalLayout_2.addWidget(self.tabWidget)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)
        self.label_2.setBuddy(self.url)
        self.label.setBuddy(self.name)
        self.label_3.setBuddy(self.updatePeriod)

        self.retranslateUi(Dialog)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QObject.connect(self.buttonBox,QtCore.SIGNAL("accepted()"),Dialog.accept)
        QtCore.QObject.connect(self.buttonBox,QtCore.SIGNAL("rejected()"),Dialog.reject)
        QtCore.QObject.connect(self.limitCount,QtCore.SIGNAL("toggled(bool)"),self.count.setEnabled)
        QtCore.QObject.connect(self.limitDays,QtCore.SIGNAL("toggled(bool)"),self.days.setEnabled)
        QtCore.QObject.connect(self.customUpdate,QtCore.SIGNAL("toggled(bool)"),self.updatePeriod.setEnabled)
        QtCore.QObject.connect(self.customUpdate,QtCore.SIGNAL("toggled(bool)"),self.updateUnit.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Dialog", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Dialog", "&URL:", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Dialog", "&Name:", None, QtGui.QApplication.UnicodeUTF8))
        self.notify.setText(QtGui.QApplication.translate("Dialog", "Notify when new articles arri&ve", None, QtGui.QApplication.UnicodeUTF8))
        self.customUpdate.setText(QtGui.QApplication.translate("Dialog", "U&se a custom update interval", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("Dialog", "Update &every", None, QtGui.QApplication.UnicodeUTF8))
        self.updateUnit.addItem(QtGui.QApplication.translate("Dialog", "Minutes", None, QtGui.QApplication.UnicodeUTF8))
        self.updateUnit.addItem(QtGui.QApplication.translate("Dialog", "Hours", None, QtGui.QApplication.UnicodeUTF8))
        self.updateUnit.addItem(QtGui.QApplication.translate("Dialog", "Days", None, QtGui.QApplication.UnicodeUTF8))
        self.updateUnit.addItem(QtGui.QApplication.translate("Dialog", "Never", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.general), QtGui.QApplication.translate("Dialog", "&General", None, QtGui.QApplication.UnicodeUTF8))
        self.useDefault.setText(QtGui.QApplication.translate("Dialog", "&Use default settings", None, QtGui.QApplication.UnicodeUTF8))
        self.keepAll.setText(QtGui.QApplication.translate("Dialog", "&Keep all articles", None, QtGui.QApplication.UnicodeUTF8))
        self.limitCount.setText(QtGui.QApplication.translate("Dialog", "Limit archi&ve to:", None, QtGui.QApplication.UnicodeUTF8))
        self.count.setSuffix(QtGui.QApplication.translate("Dialog", " articles", None, QtGui.QApplication.UnicodeUTF8))
        self.limitDays.setText(QtGui.QApplication.translate("Dialog", "&Delete older than:", None, QtGui.QApplication.UnicodeUTF8))
        self.days.setSuffix(QtGui.QApplication.translate("Dialog", " days", None, QtGui.QApplication.UnicodeUTF8))
        self.noArchive.setText(QtGui.QApplication.translate("Dialog", "Di&sable archiving", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.archive), QtGui.QApplication.translate("Dialog", "&Archive", None, QtGui.QApplication.UnicodeUTF8))
        self.loadFull.setText(QtGui.QApplication.translate("Dialog", "&Load the full website when reading articles", None, QtGui.QApplication.UnicodeUTF8))
        self.markRead.setText(QtGui.QApplication.translate("Dialog", "&Mark articles as read when they arrive", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.advanced), QtGui.QApplication.translate("Dialog", "A&dvanced", None, QtGui.QApplication.UnicodeUTF8))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'urssus/ui/searchwidget.ui'
#
# Created: Sun Jul 27 21:58:02 2008
#      by: PyQt4 UI code generator 4.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(369,32)
        self.horizontalLayout = QtGui.QHBoxLayout(Form)
        self.horizontalLayout.setMargin(2)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.close = QtGui.QToolButton(Form)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/editdelete.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.close.setIcon(icon)
        self.close.setObjectName("close")
        self.horizontalLayout.addWidget(self.close)
        self.label = QtGui.QLabel(Form)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.text = QtGui.QLineEdit(Form)
        self.text.setObjectName("text")
        self.horizontalLayout.addWidget(self.text)
        self.previous = QtGui.QToolButton(Form)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/1leftarrow.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.previous.setIcon(icon)
        self.previous.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        self.previous.setObjectName("previous")
        self.horizontalLayout.addWidget(self.previous)
        self.next = QtGui.QToolButton(Form)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/1rightarrow.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.next.setIcon(icon)
        self.next.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        self.next.setObjectName("next")
        self.horizontalLayout.addWidget(self.next)
        self.matchCase = QtGui.QCheckBox(Form)
        self.matchCase.setObjectName("matchCase")
        self.horizontalLayout.addWidget(self.matchCase)
        self.label.setBuddy(self.text)

        self.retranslateUi(Form)
        QtCore.QObject.connect(self.close,QtCore.SIGNAL("clicked()"),Form.hide)
        QtCore.QObject.connect(self.text,QtCore.SIGNAL("returnPressed()"),self.next.animateClick)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtGui.QApplication.translate("Form", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.close.setText(QtGui.QApplication.translate("Form", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.close.setShortcut(QtGui.QApplication.translate("Form", "Esc", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Form", "&Find:", None, QtGui.QApplication.UnicodeUTF8))
        self.previous.setText(QtGui.QApplication.translate("Form", "&Previous", None, QtGui.QApplication.UnicodeUTF8))
        self.next.setText(QtGui.QApplication.translate("Form", "&Next", None, QtGui.QApplication.UnicodeUTF8))
        self.matchCase.setText(QtGui.QApplication.translate("Form", "&Match Case", None, QtGui.QApplication.UnicodeUTF8))

import icons_rc



# -*- coding: utf-8 -*-

# uRSSus, a multiplatform GUI news agregator
# Copyright (C) 2008 Roberto Alsina
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


import sys, os, time, urlparse, tempfile
from urllib import urlopen
from datetime import datetime, timedelta

# Configuration
import config

# Logging
if sys.platform=='win32':
  # easylog and Processing on windows == broken
  def dumb(*a, **kw):
    print a, kw
  critical=dumb
  error=dumb
  warning=dumb
  debug=dumb
  info=dumb 
  setLogger=dumb 
  DEBUG=dumb
else:
  from easylog import critical, error, warning, debug, info, setLogger, DEBUG
  setLogger(name='urssus', level=DEBUG)

# Templates
import tenjin
# The obvious import doesn't work for complicated reasons ;-)
to_str=tenjin.helpers.to_str
escape=tenjin.helpers.escape
templateEngine=tenjin.Engine()
tmplDir=os.path.join(os.path.abspath(os.path.dirname(__file__)), 'templates')
cssFile=os.path.join(tmplDir,'style.css')

# FIXME: when deploying need to find a decent way to locate the templates
def renderTemplate(tname, **context):
  context['to_str']=to_str
  context['escape']=escape
  return templateEngine.render(os.path.join(tmplDir,tname), context)

# References to background processes
import processing
processes=[]

# A queue where subprocesses can put status messages, so 
# they appear in the window's statusbar. For example "updating NiceFeed".
statusQueue=processing.Queue()

# A queue with a list of queues to be refreshed in the feed tree
# for example when it's updated by a subprocess
# 
# [0,id] means "mark this feed as updating"
# [1,id] means "finished updating"
# [2,id,parents] means "refresh the text" (for example if you read an article, 
#                         to update the unread count,parents updates all parents)
# [3,id,count] means "notify on the systray icon that there are new articles here

feedStatusQueue=processing.Queue()

# Mark Pilgrim's feed parser
import feedparser as fp

# Some feeds put html in titles, which can't be shown in QStandardItems
from html2text import html2text as h2t

def detailToAuthor(ad):
  '''Converts something like feedparser's author_detail into a 
  nice string describing the author'''

  if 'name' in ad:
    author=ad['name']
    if 'href' in ad:
      author='<a href="%s">%s</a>'%(ad['href'], author)
  if 'email' in ad:
    email ='<a href="mailto:%s">%s</a>'%(ad['email'], ad['email'])
  else:
    email = ''

  if email and author:
    return '%s - %s'%(author, email)
  elif email:
    return email
  return author

# DB Classes
from elixir import * 
import sqlalchemy as sql

# Patch from http://elixir.ematia.de/trac/wiki/Recipes/GetByOrAddPattern
def get_by_or_init(cls, if_new_set={}, **params):
  """Call get_by; if no object is returned, initialize an
  object with the same parameters.  If a new object was
  created, set any initial values."""
  
  result = cls.get_by(**params)
  if not result:
    result = cls(**params)
    result.set(**if_new_set)
  return result

Entity.get_by_or_init = classmethod(get_by_or_init)

class Post(Entity):
  using_options (tablename='posts')
  feed        = ManyToOne('Feed')
  title       = Field(Text)
  post_id     = Field(Text)
  content     = Field(Text)
  date        = Field(DateTime)
  unread      = Field(Boolean, default=True)
  important   = Field(Boolean, default=False)
  author      = Field(Text)
  link        = Field(Text)
  deleted     = Field(Boolean, default=False)

  def __repr__(self):
    if '<' in self.title:
      return h2t(self.title).strip()
    return unicode(self.title)

class Feed(Entity):
  using_options (tablename='feeds')
  htmlUrl        = Field(Text)
  xmlUrl         = Field(Text)
  title          = Field(Text)
  text           = Field(Text, default='')
  description    = Field(Text)
  children       = OneToMany('Feed')
  parent         = ManyToOne('Feed')
  posts          = OneToMany('Post', order_by="-date")
  lastUpdated    = Field(DateTime, default=datetime(1970,1,1))
  loadFull       = Field(Boolean, default=False)
  # meaning of archiveType:
  # 0 = use default, 1 = keepall, 2 = use limitCount
  # 3 = use limitDays, 4 = no archiving
  archiveType    = Field(Integer, default=0) 
  limitCount     = Field(Integer, default=1000)
  limitDays      = Field(Integer, default=60)

  notify         = Field(Boolean, default=False)
  markRead       = Field(Boolean, default=False)
  icon           = Field(Binary, deferred=True)
  # updateInterval -1 means use the app default, any other value, it's in minutes
  updateInterval = Field(Integer, default=-1)
  curUnread      = -1

  def __repr__(self):
    c=self.unreadCount()
    if c:
      return self.text+'(%d)'%c
    return self.text

  def expire(self, expunge=False):
    '''Delete all posts that are too old'''
    # meaning of archiveType:
    # 0 = use default, 1 = keepall, 2 = use limitCount
    # 3 = use limitDays, 4 = no archiving
    now=datetime.now()
    if self.archiveType==0: # Default archive config
      for post in self.posts:
        if post.important: continue # Don't delete important stuff
        # FIXME: makethis configurable
        if (now-post.date).days>7: # Right now, keep for a week
          post.deleted=True
    elif self.archiveType==1: #keepall
      return
    elif self.archiveType==2: #limitCount
      for post in self.posts[self.limitCount:]:
        if post.important: continue # Don't delete important stuff
        post.deleted=True
    elif self.archiveType==3: #limitDays
      for post in self.posts:
        if post.important: continue # Don't delete important stuff
        if (now-post.date).days>self.limitDays: # Right now, keep for a week
          post.deleted=True
    elif self.archiveType==4: #no archiving
      for post in self.posts:
        if post.important: continue # Don't delete important stuff
        post.deleted=True
        
    session.flush()
    if expunge:
      # Delete all posts with deleted==True 
      for post in Post.query().filter(Post.feed==self).filter(Post.deleted==True).all():
        post.delete()
      session.flush()
      
    # Force recount
    self.curUnread=-1
    self.unreadCount()

  def allFeeds(self):
    '''Returns a list of all "real" feeds that have this one as ancestor'''
    if not self.children:
      return [self]
    feeds=[]
    for child in self.children:
      feeds.extend(child.allFeeds())
    return feeds

  def allPostsQuery(self):
    '''Returns a query that should give you all the posts contained in this folder's children
    down to any level. Required for aggregate feeds, I'm afraid'''
    af=self.allFeeds()
    ored=sql.or_(*[Post.feed==f for f in af])
    return Post.query().filter(ored)

  def allPosts(self):
    '''This is used if you want all posts in this feed as well as in its childrens.
    Obviously meant to be used with folders, not regular feeds ;-)
    '''
    if self.xmlUrl: #I'm not a folder
      return []
    info("allposts for feed: %s", self)
    
    # Get posts for all children
    posts=[]
    for child in self.children:
      posts.extend(child.posts)
    return posts
    
  def previousSibling(self):
    if not self.parent: return None
    sibs=self.parent.children
    ind=sibs.index(self)
    if ind==0: return None
    else:
      return sibs[ind-1]

  def nextSibling(self):
    if not self.parent: return None
    sibs=self.parent.children
    ind=sibs.index(self)+1
    if ind >= len(sibs):
      return None
    return sibs[ind]

  def lastChild(self):
    '''Goes to the last possible child of this feed (the last child of the last child ....)'''
    if not self.children:
      return self
    else:
      return self.children[-1].lastChild()

  def previousFeed(self):
    # Search for a sibling above this one, then dig
    sib=self.previousSibling()
    if sib:
      return sib.lastChild()
    else:
      # Go to parent
      if self.parent:
        return self.parent
    # We are probably at the root
    return None

  def nextFeed(self):
    # First see if we have children
    if len(self.children):
      return self.children[0]
    # Then search for a sibling below this one
    sib=self.nextSibling()
    if sib:
      return sib
    else:
      # Go to next uncle/greatuncle/whatever
      parent=self.parent
      while parent:
        nextSib=parent.nextSibling()
        if nextSib: return nextSib.nextFeed()
        parent=parent.parent
    return None

  def previousUnreadFeed(self):
    # If there are no unread articles, there is no point
    if Post.query.filter(Post.unread==True).count()==0:
      return None
      
    # First see if there is any sibling with unread items above this one
    if not self.parent: # At root feed
      return self.lastChild().previousUnreadFeed()
    sibs=self.parent.children
    sibs=sibs[:sibs.index(self)]
    sibs.reverse()
    for sib in sibs:
      if sib.unreadCount():
        if sib.xmlUrl:
          return sib
        else:
          return sib.previousUnreadFeed()
    # Then see if our parent is the answer
    if self.parent and self.parent.unreadCount():
      if self.parent.xmlUrl:
        return self.parent
      else:
        return self.parent.lastChild().previousUnreadFeed()
    elif self.parent:
      # Not him, pass the ball to uncle/gramps/whatever
      return self.parent.previousUnreadFeed()
    # Maybe should go to the *last* feed with unread articles, but it's
    # a corner case
    return None

  def nextUnreadFeed(self):
    # If there are no unread articles, there is no point
    if Post.query.filter(Post.unread==True).count()==0:
      return None
    # First see if we have children with unread articles
    if len(self.children):
      for child in self.children:
        if child.unreadCount():
          if child.xmlUrl:
            return child
          else: # Skip folders
            return child.nextUnreadFeed()
    # Then search for a sibling with unread items below this one
    sibs=self.parent.children
    for sib in sibs[sibs.index(self)+1:]:
      if sib.unreadCount():
        if sib.xmlUrl:
          return sib
        else:
          return sib.nextUnreadFeed()
    else:
      # Go to next uncle/greatuncle/whatever
      parent=self.parent
      while parent:
        nextSib=parent.nextSibling()
        if nextSib: return nextSib.nextUnreadFeed()
        parent=parent.parent
    # There is nothing below, so go to the top and try again
    return root_feed.nextUnreadFeed()
      

  def unreadCount(self):
    if self.children:
      self.curUnread=sum([ f.unreadCount() for f in self.children])
    else:
      if self.curUnread==-1:
        info ("Forcing recount in %s", self.title)
        self.curUnread=Post.query.filter(Post.feed==self).filter(Post.unread==True).count()
    return self.curUnread
      
  def updateFeedData(self):
    # assumes this feed has a xmlUrl, fetches any missing data from it
    if not self.xmlUrl: # Nowhere to fetch data from
      return
    info ( "Updating feed data from: %s", self.xmlUrl)
    d=fp.parse(self.xmlUrl)
    if not self.htmlUrl:
      self.htmlUrl=d['feed']['link']
    if not self.title:
      self.title=d['feed']['title']
      self.text=d['feed']['title']
    if not self.description:
      if 'info' in d['feed']:
        self.description=d['feed']['info']
      elif 'description' in d['feed']:
        self.description=d['feed']['description']
    if not self.icon:
      # FIXME: handle 404, 403 whatever errors
      self.icon=urlopen(urlparse.urljoin(self.htmlUrl,'/favicon.ico')).read()
      open('/tmp/icon.ico', 'w').write(self.icon)
    session.flush()

    
  def update(self):
    if not self.xmlUrl: # Not a real feed
      # FIXME: should update all children?
      return
    if self.title:
      statusQueue.put(u"Updating: "+ self.title)
    d=fp.parse(self.xmlUrl)
    posts=[]
    for post in d['entries']:
      try:
        # Date can be one of several fields
        if 'created_parsed' in post:
          dkey='created_parsed'
        elif 'published_parsed' in post:
          dkey='published_parsed'
        elif 'modified_parsed' in post:
          dkey='modified_parsed'
        else:
          dkey=None
        if dkey and post[dkey]:
          date=datetime.fromtimestamp(time.mktime(post[dkey]))
        else:
          date=datetime.now()
        
        # So can the "unique ID for this entry"
        if 'id' in post:
          idkey='id'
        elif 'link' in post:
          idkey='link'
       
        # So can the content
       
        if 'content' in post:
          content='<hr>'.join([ c.value for c in post['content']])
        elif 'summary' in post:
          content=post['summary']
        elif 'value' in post:
          content=post['value']
         
        # Rudimentary NON-html detection
        if not '<' in content:
          content='<pre>%s</pre>'%escape(content)
         
        # Author if available, else None
        author=''
        # First, we may have author_detail, which is the nicer one
        if 'author_detail' in post:
          ad=post['author_detail']
          author=detailToAuthor(ad)
        # Or maybe just an author
        elif 'author' in post:
          author=post['author']
          
        # But we may have a list of contributors
        if 'contributors' in post:
          # Which may have the same detail as the author's
          author+=' - '.join([ detailToAuthor(contrib) for contrib in post[contributors]])
        if not author:
          #FIXME: how about using the feed's author, or something like that
          author=None
          
        # The link should be simple ;-)
        if 'link' in post:
          link=post['link']
        else:
            link=None
        # FIXME: if I use date to check here, I get duplicates on posts where I use
        # artificial date because it's not in the feed's entry.
        # If I don't I don't re-get updated posts.
        p = Post.get_by(feed=self, title=post['title'],post_id=post[idkey])
        if not p:
          p=Post(feed=self, date=date, title=post['title'], 
                 post_id=post[idkey], content=content, 
                 author=author, link=link)
          if self.markRead:
            p.unread=False
          posts.append(p)
      except KeyError:
        debug( post )
    self.lastUpdated=datetime.now()
    session.flush()
      
    # Queue a notification if needed
    if posts and self.notify:
      feedStatusQueue.put([3, self.id, len(posts)])

  def getQuery(self):
    if self.xmlUrl:
      return Post.query.filter(Post.feed==self)
    else:
      return self.allPostsQuery()

  def nextPost(self, post, order, required=None, textFilter=''):
    '''Returns next post in this feed after "post" or None'''
    posts=self.getQuery()
    if required:
      posts=posts.filter(sql.or_(Post.id==post.id, required==True))
    if textFilter:
      posts=posts.filter(sql.or_(Post.id==post.id,Post.title.like('%%%s%%'%textFilter), Post.content.like('%%%s%%'%textFilter)))
    posts=posts.order_by(order).all()
    if posts:
      ind=posts.index(post)
      if ind+1<len(posts):
        return posts[ind+1]
    return None

  def nextUnreadPost(self, post, order, required=None, textFilter=''):
    '''Returns next unread post after "post" in this feed or None'''
    posts=self.getQuery().filter(sql.or_(Post.unread==True, Post.id==post.id))
    if required:
      posts=posts.filter(sql.or_(Post.id==post.id, required==True))
    if textFilter:
      posts=posts.filter(sql.or_(Post.id==post.id,Post.title.like('%%%s%%'%textFilter), Post.content.like('%%%s%%'%textFilter)))
    posts=posts.order_by(order).all()    
    if posts:
      ind=posts.index(post)
      if ind+1<len(posts):
        return posts[ind+1]
    return None

  def previousPost(self, post, order, required=None, textFilter=''):
    '''Returns previous post in this feed or None'''
    posts=self.getQuery()
    if required:
      posts=posts.filter(sql.or_(Post.id==post.id, required==True))
    if textFilter:
      posts=posts.filter(sql.or_(Post.id==post.id,Post.title.like('%%%s%%'%textFilter), Post.content.like('%%%s%%'%textFilter)))
    posts=posts.order_by(order).all()
    if posts:
      ind=posts.index(post)
      if ind>0:
        return posts[ind-1]
    return None

  def previousUnreadPost(self, post, order, required=None, textFilter=''):
    '''Returns previous post in this feed or None'''
    posts=self.getQuery().filter(sql.or_(Post.unread==True, Post.id==post.id))
    if required:
      posts=posts.filter(sql.or_(Post.id==post.id, required==True))
    if textFilter:
      posts=posts.filter(sql.or_(Post.id==post.id,Post.title.like('%%%s%%'%textFilter), Post.content.like('%%%s%%'%textFilter)))
    posts=posts.order_by(order).all()
    if posts:
      ind=posts.index(post)
      if ind>0:
        return posts[ind-1]
    return None

root_feed=None

def initDB():
  global root_feed
  # This is just temporary
  metadata.bind = "sqlite:///%s/urssus.sqlite"%config.cfdir
  # metadata.bind.echo = True
  setup_all()
  if not os.path.exists("urssus.sqlite"):
    create_all()
  root_feed=Feed.get_by_or_init(parent=None)
  session.flush()

# UI Classes
from PyQt4 import QtGui, QtCore, QtWebKit
from ui.Ui_main import Ui_MainWindow
from ui.Ui_about import Ui_Dialog as UI_AboutDialog
from ui.Ui_filterwidget import Ui_Form as UI_FilterWidget
from ui.Ui_searchwidget import Ui_Form as UI_SearchWidget
from ui.Ui_feed_properties import Ui_Dialog as UI_FeedPropertiesDialog

class PostModel(QtGui.QStandardItemModel):
  def __init__(self):
    QtGui.QStandardItemModel.__init__(self)
    self.setColumnCount(2)
    self.setHeaderData(0, QtCore.Qt.Horizontal, QtCore.QVariant("Title"))
    self.setHeaderData(1, QtCore.Qt.Horizontal, QtCore.QVariant("Date"))
    self.sort(1, QtCore.Qt.DescendingOrder) # Date, descending

  def sortOrder(self):
    order=["title", "date"][self.sortingColumn]
    if self.order==QtCore.Qt.DescendingOrder:
      order=sql.desc(order)
    info("ORDER: %s", order)
    return order

  def sort(self, column, order):
    info ("Changing post model sort order to %d %d", column, order)
    self.sortingColumn=column
    self.order=order
    self.emit(QtCore.SIGNAL("resorted()"))

  def data(self, index, role):
    if not index.isValid():
      return QtCore.QVariant()
      
    if role<>QtCore.Qt.DisplayRole:
      return QtGui.QStandardItemModel.data(self, index, role)
    
    if index.column()==0:
      item=self.itemFromIndex(index)
      v=QtCore.QVariant(unicode(item.post))
    elif index.column()==1:
      # Tricky!
      ind=self.index(index.row(), 0, index.parent())
      item=self.itemFromIndex(ind)
      v=QtCore.QVariant(str(item.post.date))
    else:
      return QtCore.QVariant()
    return v
    
class FilterWidget(QtGui.QWidget):
  def __init__(self):
    QtGui.QWidget.__init__(self)
    # Set up the UI from designer
    self.ui=UI_FilterWidget()
    self.ui.setupUi(self)
    
class SearchWidget(QtGui.QWidget):
  def __init__(self):
    QtGui.QWidget.__init__(self)
    # Set up the UI from designer
    self.ui=UI_SearchWidget()
    self.ui.setupUi(self)

class AboutDialog(QtGui.QDialog):
  def __init__(self):
    QtGui.QDialog.__init__(self)
    # Set up the UI from designer
    self.ui=UI_AboutDialog()
    self.ui.setupUi(self)

class TrayIcon(QtGui.QSystemTrayIcon):
  def __init__(self):
    QtGui.QSystemTrayIcon.__init__ (self,QtGui.QIcon(":/urssus.svg"))
 
class FeedProperties(QtGui.QDialog):
  def __init__(self, feed):
    QtGui.QDialog.__init__(self)
    # Set up the UI from designer
    self.ui=UI_FeedPropertiesDialog()
    self.ui.setupUi(self)
    self.ui.tabWidget.setCurrentIndex(0)
    self.feed=feed
    self.loadData()
  
  def loadData(self):
    feed=self.feed
    self.ui.name.setText(feed.text)
    self.ui.url.setText(feed.xmlUrl)
    self.ui.notify.setChecked(feed.notify)
    if feed.updateInterval==-1: # Use default
      self.ui.updatePeriod.setEnabled(False)
      self.ui.updateUnit.setEnabled(False)
      self.ui.customUpdate.setChecked(False)
    else:
      self.ui.updatePeriod.setEnabled(True)
      self.ui.updateUnit.setEnabled(True)
      self.ui.customUpdate.setChecked(True)
      if feed.updateInterval==0:
        i=3
      elif feed.updateInterval%1440==0:
        i=2
      elif feed.updateInterval%60==0:
        i=1
      else:
        i=0
      self.ui.updateUnit.setCurrentIndex(i)
      self.ui.updatePeriod.setValue(feed.updateInterval/([1, 60, 1440, 1][i]))
      
    if feed.archiveType==0: # Use default archiving
      self.ui.useDefault.setChecked(True)
    elif feed.archiveType==1: # Keep all articles
      self.ui.keepAll.setChecked(True)
    elif feed.archiveType==2: # Keep a number of articles
      self.ui.limitCount.setChecked(True)
      self.ui.count.setValue(feed.limitCount)
    elif feed.archiveType==3: # Keep for a period of time
      self.ui.limitDays.setChecked(True)
      self.ui.days.setValue(feed.limitDays)
    elif feed.archiveType==4: # Keep nothing
      self.ui.noArchive.setChecked(True)
  
    self.ui.loadFull.setChecked(feed.loadFull)
    self.ui.markRead.setChecked(feed.markRead)
    
  def accept(self):
    feed=self.feed
    feed.text=unicode(self.ui.name.text())
    # FIXME: validate
    feed.xmlUrl=unicode(self.ui.url.text())
    feed.notify=self.ui.notify.isChecked()
    feed.markRead=self.ui.markRead.isChecked()
    feed.loadFull=self.ui.loadFull.isChecked()
    
    if self.ui.customUpdate.isChecked():
      multiplier=[1, 60, 1440, 0][self.ui.updateUnit.currentIndex()]
      feed.updateInterval=self.ui.updatePeriod.value()*multiplier

    if self.ui.useDefault.isChecked(): # Default expire
      feed.archiveType=0
    elif self.ui.keepAll.isChecked(): # Keep everything
      feed.archiveType=1
    elif self.ui.limitCount.isChecked(): # Limit by count
      feed.archiveType=2
      feed.limitCount=self.ui.count.value()
    elif self.ui.limitDays.isChecked(): # Limit by age
      feed.archiveType=3
      feed.limitDays=self.ui.days.value()
    elif self.ui.noArchive.isChecked(): # Don't archive
      feed.archiveType=4


    session.flush()
    QtGui.QDialog.accept(self)
 
class MainWindow(QtGui.QMainWindow):
  def __init__(self):
    QtGui.QMainWindow.__init__(self)

    # Internal indexes
    self.feedItems={}
    self.postItems={}
    self.posts=[]
    self.currentFeed=None
    self.currentPost=None
    
    # View mode FIXME: make configurable
    self.combinedView=False
    
    # Set up the UI from designer
    self.ui=Ui_MainWindow()
    self.ui.setupUi(self)
    
    # add widgets to status bar
    self.progress=QtGui.QProgressBar()
    self.progress.setFixedWidth(120)
    self.ui.statusBar.addPermanentWidget(self.progress)
    
    # Use custom delegate to paint feed and post items
    self.ui.feeds.setItemDelegate(FeedDelegate(self))
    self.ui.posts.setItemDelegate(PostDelegate(self))

    # Article filter fields
    self.filterWidget=FilterWidget()
    self.ui.filterBar.addWidget(self.filterWidget)
    QtCore.QObject.connect(self.filterWidget.ui.filter, QtCore.SIGNAL("returnPressed()"), self.filterPosts)
    QtCore.QObject.connect(self.filterWidget.ui.clear, QtCore.SIGNAL("clicked()"), self.unFilterPosts)
    QtCore.QObject.connect(self.filterWidget.ui.statusCombo, QtCore.SIGNAL("currentIndexChanged(int)"), self.filterPostsByStatus)
    self.statusFilter=None
    self.textFilter=''
        
    # Search widget
    self.searchWidget=SearchWidget()
    self.searchWidget.hide()
    self.ui.splitter.addWidget(self.searchWidget)
    QtCore.QObject.connect(self.searchWidget.ui.next, QtCore.SIGNAL("clicked()"), self.findText)
    QtCore.QObject.connect(self.searchWidget.ui.previous, QtCore.SIGNAL("clicked()"), self.findTextReverse)
    
    # Set some properties of the Web view
    page=self.ui.view.page()
    page.setLinkDelegationPolicy(page.DelegateAllLinks)
    self.ui.view.setFocus(QtCore.Qt.TabFocusReason)
    QtWebKit.QWebSettings.globalSettings().setUserStyleSheetUrl(QtCore.QUrl(cssFile))
    QtCore.QObject.connect(self.ui.view.page(), QtCore.SIGNAL(" linkHovered ( const QString & link, const QString & title, const QString & textContent )"), self.linkHovered)

    # Set sorting for post list
    self.ui.posts.sortByColumn(1, QtCore.Qt.DescendingOrder)

    # Fill with feed data
    self.showOnlyUnread=False
    self.initTree()

    # Timer to trigger status bar updates
    self.statusTimer=QtCore.QTimer()
    self.statusTimer.setSingleShot(True)
    QtCore.QObject.connect(self.statusTimer, QtCore.SIGNAL("timeout()"), self.updateStatusBar)
    self.statusTimer.start(0)
    
    # Timer to mark feeds as busy/updated/whatever
    self.feedStatusTimer=QtCore.QTimer()
    self.feedStatusTimer.setSingleShot(True)
    QtCore.QObject.connect(self.feedStatusTimer, QtCore.SIGNAL("timeout()"), self.updateFeedStatus)
    self.feedStatusTimer.start(0)
    self.updatesCounter=0

    # Load user preferences
    self.loadPreferences()

    # Tray icon
    self.tray=TrayIcon()
    self.tray.show()
    self.notifiedFeed=None
    QtCore.QObject.connect(self.tray, QtCore.SIGNAL("messageClicked()"), self.notificationClicked)

  def notificationClicked(self):
    if self.notifiedFeed:
      self.open_feed(self.ui.feeds.model().indexFromItem(self.feedItems[self.notifiedFeed.id]))

  def loadPreferences(self):
    
    v=config.getValue('ui', 'showStatus', True)
    self.ui.actionStatus_Bar.setChecked(v)
    self.on_actionStatus_Bar_triggered(v)
    
    v=config.getValue('ui', 'showOnlyUnreadFeeds', False)
    self.ui.actionShow_Only_Unread_Feeds.setChecked(v)
    self.on_actionShow_Only_Unread_Feeds_triggered(v)

  def getCurrentPost(self):
    index=self.ui.posts.currentIndex()
    if index.isValid():         
      if index.column()<>0:
        index=self.ui.posts.model().index(index.row(), 0, index.parent())
      return self.ui.posts.model().itemFromIndex(index).post
    return None

  def on_actionDelete_Article_triggered(self, i=None):
    # FIXME: handle selections
    if i==None: return
    curPost=self.getCurrentPost()
    if not curPost: return
    info ("Deleting post: %s", curPost)
    curPost.delete()
    session.flush()
    self.open_feed(self.ui.feeds.currentIndex())

  def on_actionMark_as_Read_triggered(self, i=None):
    # FIXME: handle selections
    if i==None: return
    curPost=self.getCurrentPost()
    if not curPost: return
    if curPost.unread:
      info ("Marking as read post: %s", curPost)
      curPost.unread=False
      curPost.feed.curUnread-=1 
      session.flush()
      self.updatePostItem(curPost)
      self.updateFeedItem(curPost.feed)

  def on_actionMark_as_Unread_triggered(self, i=None):
    # FIXME: handle selections
    if i==None: return
    curPost=self.getCurrentPost()
    if not curPost: return
    if not curPost.unread:
      info ("Marking as unread post: %s", curPost)
      curPost.unread=True
      curPost.feed.curUnread+=1
      session.flush()
      self.updatePostItem(curPost)
      self.updateFeedItem(curPost.feed)

  def on_actionOpen_in_Browser_triggered(self, i=None):
    # FIXME: handle selections
    if i==None: return
    curPost=self.getCurrentPost()
    if not curPost: return
    QtGui.QDesktopServices.openUrl(QtCore.QUrl(curPost.link))


  def on_actionMark_as_Important_triggered(self, i=None):
    # FIXME: handle selections
    if i==None: return
    curPost=self.getCurrentPost()
    if not curPost: return
    info ("Marking as important post: %s", curPost)
    curPost.important=True
    session.flush()
    self.updatePostItem(curPost)

  def on_actionRemove_Important_Mark_triggered(self, i=None):
    # FIXME: handle selections
    if i==None: return
    curPost=self.getCurrentPost()
    if not curPost: return
    info ("Marking as not important post: %s", curPost)
    curPost.important=False
    session.flush()
    self.updatePostItem(curPost)


  def on_posts_customContextMenuRequested(self, pos=None):
    # FIXME: handle selections
    if pos==None: return
    curPost=self.getCurrentPost()
    if not curPost: return
    menu=QtGui.QMenu()
    menu.addAction(self.ui.actionOpen_in_Browser)
    menu.addSeparator()
    if curPost.important:
      menu.addAction(self.ui.actionRemove_Important_Mark)
    else:
      menu.addAction(self.ui.actionMark_as_Important)
    if curPost.unread:
      menu.addAction(self.ui.actionMark_as_Read)
    else:
      menu.addAction(self.ui.actionMark_as_Unread)
    menu.addAction(self.ui.actionDelete_Article)
    menu.exec_(QtGui.QCursor.pos())

  def on_feeds_customContextMenuRequested(self, pos=None):
    if pos==None: return
    
    item=self.model.itemFromIndex(self.ui.feeds.currentIndex())
    if item and item.feed:
      menu=QtGui.QMenu()
      menu.addAction(self.ui.actionMark_Feed_as_Read)
      menu.addSeparator()
      menu.addAction(self.ui.actionFetch_Feed)
      menu.addSeparator()
      menu.addAction(self.ui.actionOpen_Homepage)
      menu.addSeparator()
      menu.addAction(self.ui.actionEdit_Feed)
      menu.addAction(self.ui.actionExpire_Feed)
      menu.addAction(self.ui.actionDelete_Feed)
      menu.exec_(QtGui.QCursor.pos())

  def on_actionExpire_Feed_triggered(self, i=None):
    if i==None: return
    index=self.ui.feeds.currentIndex()
    if index.isValid():         
      curFeed=self.ui.feeds.model().itemFromIndex(index).feed
    info ("Expiring feed: %s", curFeed)
    curFeed.expire(expunge=True)
    # Update feed display (number of unreads may have changed)
    self.updateFeedItem(curFeed)
    # Reopen it because the post list probably changed
    self.open_feed(index)

  def on_actionEdit_Feed_triggered(self, i=None):
    if i==None: return
    index=self.ui.feeds.currentIndex()
    if index.isValid():         
      curFeed=self.ui.feeds.model().itemFromIndex(index).feed
    info ("Editing feed: %s", curFeed)

    editDlg=FeedProperties(curFeed)
    if editDlg.exec_():
      # update feed item
      self.updateFeedItem(curFeed)

  def on_actionAdd_Feed_triggered(self, i=None):
    if i==None: return
    # Ask for feed URL
    [url, ok]=QtGui.QInputDialog.getText(self, "Add Feed - uRSSus", "&Feed URL:")
    if ok:
      url=unicode(url)
      # Use Mark pilgrim / Aaron Swartz's RSS finder module
      # FIXME: make this non-blocking somehow
      import feedfinder
      feed=feedfinder.feed(url)
      if not feed:
        QtGui.QMessageBox.critical(self, "Error - uRSSus", "Can't find a feed wit URL: %s"%url)
        return
      info ("Found feed: %s", feed)
      if Feed.get_by(xmlUrl=feed):
        # Already subscribed
        QtGui.QMessageBox.critical(self, "Error - uRSSus", "You are already subscribed")
        return
      newFeed=Feed(xmlUrl=feed)
      newFeed.updateFeedData()
      
      # Figure out the insertion point
      index=self.ui.feeds.currentIndex()
      if index.isValid():         
        curFeed=self.ui.feeds.model().itemFromIndex(index).feed
      else:
        curFeed=root_feed
      # if curFeed is a feed, add as sibling
      if curFeed.xmlUrl:
        newFeed.parent=curFeed.parent
      # if curFeed is a folder, add as child
      else:
        newFeed.parent=curFeed
      session.flush()
      self.initTree()
      self.ui.feeds.setCurrentIndex(self.ui.feeds.model().indexFromItem(self.feedItems[newFeed.id]))
      self.on_actionEdit_Feed_triggered(True)

  def on_actionNew_Folder_triggered(self, i=None):
    if i==None: return
    # Ask for folder name
    [name, ok]=QtGui.QInputDialog.getText(self, "Add Folder - uRSSus", "&Folder name:")
    if ok:
      newFolder=Feed(text=unicode(name))
      # Figure out the insertion point
      index=self.ui.feeds.currentIndex()
      if index.isValid():         
        curFeed=self.itemFromIndex(index).feed
      else:
        curFeed=root_feed
      # if curFeed is a feed, add as sibling
      if curFeed.xmlUrl:
        newFolder.parent=curFeed.parent
      # if curFeed is a folder, add as child
      else:
        newFolder.parent=curFeed
      session.flush()
      self.initTree()
      self.ui.feeds.setCurrentIndex(self.ui.feeds.model().indexFromItem(self.feedItems[newFolder.id]))

  def on_actionShow_Only_Unread_Feeds_triggered(self, checked=None):
    if checked==None: return
    info ("Show only unread: %d", checked)
    self.showOnlyUnread=checked
    for feed in Feed.query().all():
      self.updateFeedItem(feed)
    config.setValue('ui', 'showOnlyUnreadFeeds', checked)
  
  def on_actionFind_triggered(self, i=None):
    if i==None: return
    self.searchWidget.show()
    self.searchWidget.ui.text.setFocus(QtCore.Qt.TabFocusReason)

  def on_actionFind_Again_triggered(self, i=None):
    if i==None: return
    self.findText()

  def findText(self):
    text=unicode(self.searchWidget.ui.text.text())
    if self.searchWidget.ui.matchCase.isChecked():
      self.ui.view.findText(text, QtWebKit.QWebPage.FindCaseSensitively)
    else:  
      self.ui.view.findText(text)

  def findTextReverse(self):
    text=unicode(self.searchWidget.ui.text.text())
    if self.searchWidget.ui.matchCase.isChecked():
      self.ui.view.findText(text, QtWebKit.QWebPage.FindBackward | QtWebKit.QWebPage.FindCaseSensitively)
    else:  
      self.ui.view.findText(text, QtWebKit.QWebPage.FindBackward)

  def filterPostsByStatus(self, status=None):
    if status==None: return
    
    if status==0: # All articles
      info ("No filtering by status")
      self.statusFilter=None
    elif status==1: # Unread
      info ("Filtering by status: unread")
      self.statusFilter=Post.unread
    elif status==2: # Important
      info ("Filtering by status: important")
      self.statusFilter=Post.important
    self.open_feed(self.ui.feeds.currentIndex())
      
  def unFilterPosts(self):
    self.textFilter=''
    info("Text filter removed")
    self.open_feed(self.ui.feeds.currentIndex())

  def filterPosts(self):
    self.textFilter=unicode(self.filterWidget.ui.filter.text())
    info("Text filter set to: %s", self.textFilter)
    self.open_feed(self.ui.feeds.currentIndex())

  def linkHovered(self, link, title, content):
    # FIXME: doesn't trigger. Maybe I need to reconnect after 
    # I set the contents of the webview?
    self.ui.statusBar.showMessage(link)

  def on_view_linkClicked(self, url):
    QtGui.QDesktopServices.openUrl(url)

  def on_view_loadStarted(self):
    self.progress.show()
    self.progress.setValue(0)

  def on_view_loadProgress(self, p):
    self.progress.setValue(p)

  def on_view_loadFinished(self):
    self.progress.setValue(0)
    self.progress.hide()

  def on_actionStatus_Bar_triggered(self, i=None):
    if i==None: return
    if self.ui.actionStatus_Bar.isChecked():
      self.statusBar().show()
    else:
      self.statusBar().hide()
    config.setValue('ui', 'showStatus', self.ui.actionStatus_Bar.isChecked())

  def on_actionAbout_uRSSus_triggered(self, i=None):
    if i==None: return
    AboutDialog().exec_()
    
  def updateFeedStatus(self):
    while not feedStatusQueue.empty():
      data=feedStatusQueue.get()
      [action, id] = data[:2]
      info("updateFeedStatus: %d %d", action, id)
      if not id in self.feedItems:
        # This shouldn't happen, it means there is a 
        # feed that is not in the tree
        error( "id %s not in the tree", id)
        sys.exit(1)
      feed=Feed.get_by(id=id)
      if action==0: # Mark as updating
        self.updateFeedItem(feed, updating=True)
        self.updatesCounter+=1
      elif action==1: # Mark as finished updating
        # Force recount after update
        feed.curUnread=-1
        self.updateFeedItem(feed, updating=False, parents=True)
        self.updatesCounter-=1
      elif action==2: # Just update it
        self.updateFeedItem(feed, data[2])
      elif action==3: # Systray notification
        self.notifiedFeed=feed
        self.tray.showMessage("New Articles", "%d new articles in %s"%(data[2], feed.text) )
      if self.updatesCounter>0:
        self.ui.actionAbort_Fetches.setEnabled(True)
      else:
        self.ui.actionAbort_Fetches.setEnabled(False)

    self.feedStatusTimer.start(1000)

  def updateStatusBar(self):
    if not statusQueue.empty():
      msg=statusQueue.get()
      self.statusBar().showMessage(msg)
    else:
      self.statusBar().showMessage("")      
    if statusQueue.empty():
      self.statusTimer.start(1000)
    else:
      self.statusTimer.start(100)

  def initTree(self):
    # Initialize the tree from the Feeds
    self.model=QtGui.QStandardItemModel()
    self.ui.feeds.setModel(self.model)
    self.feedItems={}
    
    # Internal function
    def addSubTree(parent, node):
      nn=QtGui.QStandardItem(unicode(node))
      nn.setToolTip(unicode(node))
      parent.appendRow(nn)
      nn.feed=node
      self.feedItems[node.id]=nn
      self.updateFeedItem(node)
      if node.xmlUrl:
        nn.setIcon(QtGui.QIcon(":/urssus.svg"))
      else:
        nn.setIcon(QtGui.QIcon(":/folder.svg"))
        for child in node.children:
          addSubTree(nn, child)
      return nn
          
    iroot=self.model.invisibleRootItem()
    iroot.feed=root_feed
    self.feedItems[root_feed.id]=iroot
    for root in root_feed.children:
      addSubTree(iroot, root)
      
    self.ui.feeds.expandAll()
    
  def on_feeds_clicked(self, index):
    item=self.model.itemFromIndex(index)
    if not item: return
    self.open_feed(index)
    if self.combinedView:
      self.open_feed(index)
    else:
      self.ui.view.setHtml(renderTemplate('feed.tmpl', feed=item.feed))

  def on_actionNormal_View_triggered(self, i=None):
    if i==None: return
    info("Switch to normal view")
    if self.combinedView:
      self.combinedView=False
      self.ui.posts.show()
      self.ui.actionNormal_View.setEnabled(False)
      self.ui.actionCombined_View.setEnabled(True)
      self.open_feed(self.ui.feeds.currentIndex())

  def on_actionCombined_View_triggered(self, i=None):
    if i==None: return
    info("Switch to combined view")    
    if not self.combinedView:
      self.combinedView=True
      self.ui.posts.hide()
      self.ui.actionNormal_View.setEnabled(True)
      self.ui.actionCombined_View.setEnabled(False)
      self.open_feed(self.ui.feeds.currentIndex())

  def resortPosts(self):
    info ("Resorting posts")
    if self.currentPost:
      cpid=self.currentPost.id
      info ("cpid=%d", cpid)
    else:
      cpid=-1
    self.open_feed(self.ui.feeds.currentIndex())
    if cpid in self.postItems:
      self.ui.posts.setCurrentIndex(self.ui.posts.model().indexFromItem(self.postItems[cpid]))
      self.currentPost=Post.get_by(id=cpid)

  def open_feed(self, index):
    item=self.model.itemFromIndex(index)
    if not item: return
    self.ui.feeds.setCurrentIndex(index)
    feed=item.feed
    self.currentFeed=feed
    self.postItems={}
    self.posts=[]
    self.currentPost=None
    
    actions=[ self.ui.actionNext_Article,
              self.ui.actionNext_Unread_Article,  
              self.ui.actionPrevious_Article,  
              self.ui.actionPrevious_Unread_Article,  
              self.ui.actionMark_as_Read,  
              self.ui.actionMark_as_Unread,  
              self.ui.actionMark_as_Important,  
              self.ui.actionDelete_Article,  
              self.ui.actionOpen_in_Browser,  
              self.ui.actionRemove_Important_Mark,  
             ]
    
    if self.combinedView: # All items filtered, as a page
      info("Opening combined")
      if feed.xmlUrl: # A regular feed
        self.posts=Post.query.filter(Post.feed==feed)
      else: # A folder
        self.posts=feed.allPostsQuery()
      # Filter by text according to the contents of self.textFilter
      if self.textFilter:
        self.posts=self.posts.filter(sql.or_(Post.title.like('%%%s%%'%self.textFilter), Post.content.like('%%%s%%'%self.textFilter)))
      if self.statusFilter:
        self.posts=self.posts.filter(self.statusFilter==True)
      # FIXME: find a way to add sorting to the UI for this (not very important)
      self.posts=self.posts.order_by(sql.desc(Post.date)).all()
      self.ui.view.setHtml(renderTemplate('combined.tmpl',posts=self.posts))
      for post in self.posts:
        self.postItems[post.id]=item
        
      for action in actions:
        action.setEnabled(False)

    else: # Standard View
      info ("Opening in standard view")
      self.ui.posts.__model=PostModel()
      self.ui.posts.setModel(self.ui.posts.__model)
      QtCore.QObject.connect(self.ui.posts.__model, QtCore.SIGNAL("resorted()"), self.resortPosts)

      # Update window title
      if feed.title:
        self.setWindowTitle("%s - uRSSus"%item.feed.title)
      elif feed.text:
        self.setWindowTitle("%s - uRSSus"%item.feed.text)
      else:
        self.setWindowTitle("uRSSus")
      
      # Sorting according to the model
      sk=self.ui.posts.model().sortOrder()
      
      if feed.xmlUrl: # A regular feed
        self.posts=Post.query.filter(Post.feed==feed)
      else: # A folder
        self.posts=feed.allPostsQuery()
      # Filter by text according to the contents of self.textFilter
      if self.textFilter:
        self.posts=self.posts.filter(sql.or_(Post.title.like('%%%s%%'%self.textFilter), Post.content.like('%%%s%%'%self.textFilter)))
      if self.statusFilter:
        self.posts=self.posts.filter(self.statusFilter==True)
      self.posts=self.posts.order_by(sk)
      self.posts=self.posts.all()
    
      # Fixes for post list UI
      header=self.ui.posts.header()
      header.setStretchLastSection(False)
      header.setResizeMode(0, QtGui.QHeaderView.Stretch)
      header.setResizeMode(1, QtGui.QHeaderView.Fixed)
      header.resizeSection(1, header.fontMetrics().width(' 8888-88-88 88:88:88 ')+4)
    
      for post in self.posts:
        item=QtGui.QStandardItem('%s - %s'%(decodeString(post.title), post.date))
        item.post=post
        self.ui.posts.__model.appendRow(item)
        self.postItems[post.id]=item
        self.updatePostItem(post)

      for action in actions:
        action.setEnabled(True)

      self.ui.view.setHtml(renderTemplate('feed.tmpl',feed=feed))
      # Scroll post view to the top
      if self.posts:
        self.ui.posts.scrollTo(self.ui.posts.model().indexFromItem(self.postItems[self.posts[0].id]))


  def updatePostItem(self, post):
    if not post.id in self.postItems: #post is not being displayed
      return
      
    if self.combinedView: # The post items are not visible anyway
      return
      
    item=self.postItems[post.id]
    index=self.ui.posts.model().indexFromItem(item)
    item2=self.ui.posts.model().itemFromIndex(self.ui.posts.model().index(index.row(), 1, index.parent()))
    # FIXME: respect the palette
    if post.important:
      item.setForeground(QtGui.QColor("red"))
      item2.setForeground(QtGui.QColor("red"))
    elif post.unread:
      item.setForeground(QtGui.QColor("darkgreen"))
      item2.setForeground(QtGui.QColor("darkgreen"))
    else:
      item.setForeground(QtGui.QColor("black"))
      item2.setForeground(QtGui.QColor("black"))

  def updateFeedItem(self, feed, parents=False, updating=False):
    info("Updating item for feed %d", feed.id)
    if not feed.id in self.feedItems:
      return
    item=self.feedItems[feed.id]
    # The calls to setRowHidden cause a change in the column's width! Looks like a Qt bug to me.
    if self.showOnlyUnread:
      if feed.unreadCount()==0 and feed<>self.currentFeed: 
        # Hide feeds with no unread items
        self.ui.feeds.setRowHidden(item.row(), self.model.indexFromItem(item.parent()), True)
      else:
        self.ui.feeds.setRowHidden(item.row(), self.model.indexFromItem(item.parent()), False)
    else:
      if self.ui.feeds.isRowHidden(item.row(), self.model.indexFromItem(item.parent())):
        self.ui.feeds.setRowHidden(item.row(), self.model.indexFromItem(item.parent()), False)
    if updating:
      item.setForeground(QtGui.QColor("darkgrey"))
    else:
      item.setForeground(QtGui.QColor("black"))
    item.setText(unicode(feed))
    if parents: # Not by default because it's slow
      # Update all ancestors too, because unread counts and such change
      while feed.parent:
        self.updateFeedItem(feed.parent, True)
        feed=feed.parent

  def on_posts_clicked(self, index=None, item=None):
    if item: post=item.post
    else: 
      if index.column()<>0:
        index=self.ui.posts.model().index(index.column(), 0, index.parent())
      post=self.ui.posts.model().itemFromIndex(index).post
    self.currentPost=post
    if post.unread:
      post.unread=False
      post.feed.curUnread-=1
      session.flush()
    self.updateFeedItem(post.feed, parents=True)
    self.updatePostItem(post)
    if post.feed.loadFull and post.link:
      # If I pass post.link, it crashes if I click something else quickly
      self.ui.statusBar.showMessage("Opening %s"%post.link)
      self.ui.view.setUrl(QtCore.QUrl(QtCore.QString(post.link)))
    else:
      self.ui.view.setHtml(renderTemplate('post.tmpl',post=post))

  def on_actionExport_Feeds_triggered(self, i=None):
    if i==None: return
    fname = unicode(QtGui.QFileDialog.getSaveFileName(self, "Save as", os.getcwd(), 
                                              "OPML files (*.opml *.xml)"))
    if fname:
      exportOPML(fname)

  def on_actionImport_Feeds_triggered(self, i=None):
    if i==None: return
    fname = unicode(QtGui.QFileDialog.getOpenFileName(self, "Open OPML file", os.getcwd(), 
                                              "OPML files (*.opml *.xml)"))
    if fname:
      importOPML(fname)
      self.initTree()
      
  def on_actionTechnorati_Top_100_triggered(self, i=None):
    if i==None: return
    if QtGui.QMessageBox.question(None, "Technorati Top 100 - uRSSus", 
       'You are about to import Technorati\'s Top 100 feeds for today.\nClick Yes to confirm.', 
       QtGui.QMessageBox.Yes | QtGui.QMessageBox.No, QtGui.QMessageBox.No ) == QtGui.QMessageBox.Yes:
      
      url='http://elliottback.com/tools/top100/technorati-100-to-opml.php'
      data=urlopen(url).read()
      # Create a 'Top 100' folder if there isn't one
      t100=Feed.get_by_or_init(text='Top 100')
      if not t100.parent:
        t100.parent=root_feed
      [h, name]=tempfile.mkstemp()
      tf=os.fdopen(h, 'wb+')
      tf.write(data)
      tf.flush()
      tf.close()
      importOPML(name, parent=t100)
      os.unlink(name)
      self.initTree()
      self.ui.feeds.setCurrentIndex(self.ui.feeds.model().indexFromItem(self.feedItems[t100.id]))
    
      
  def on_actionQuit_triggered(self, i=None):
    if i==None: return
    QtGui.QApplication.instance().quit()

  def on_actionMark_Feed_as_Read_triggered(self, i=None):
    if i==None: return
    item=self.model.itemFromIndex(self.ui.feeds.currentIndex())
    if item and item.feed:
      for post in item.feed.posts:
        if post.unread:
          post.unread=False
          post.feed.curUnread-=1
          self.updatePostItem(post)
      session.flush()
      self.updateFeedItem(item.feed, parents=True)

  def on_actionDelete_Feed_triggered(self, i=None):
    if i==None: return
    index=self.ui.feeds.currentIndex()
    item=self.model.itemFromIndex(index)
    if item and item.feed:
      info( "Deleting %s", item.feed)
      if QtGui.QMessageBox.question(None, "Delete Feed - uRSSus", 
         'Are you sure you want to delete "%s"'%item.feed, 
         QtGui.QMessageBox.Yes | QtGui.QMessageBox.No, QtGui.QMessageBox.No ) == QtGui.QMessageBox.Yes:
        parent=item.feed.parent

        # Clean posts list
        self.ui.posts.setModel(PostModel())
        self.ui.view.setHtml('')

        # Trigger update on parent item
        parent.curUnread=-1
        # I really, really shouldn't have to do this. But it doesn'twork if I don't so...
        parent.children.remove(item.feed)
        self.updateFeedItem(parent, parents=True)

        # No feed current
        self.ui.feeds.setCurrentIndex(QtCore.QModelIndex())
        self.currentFeed=None

        del(self.feedItems[item.feed.id])
        item.feed.delete()
        self.ui.feeds.model().removeRow(index.row(), index.parent())
        session.flush()
        
  def on_actionOpen_Homepage_triggered(self, i=None):
    if i==None: return
    item=self.model.itemFromIndex(self.ui.feeds.currentIndex())
    if item and item.feed and item.feed.htmlUrl:
      info("Opening %s", item.feed.htmlUrl)
      QtGui.QDesktopServices.openUrl(QtCore.QUrl(item.feed.htmlUrl))
    

  def on_actionFetch_Feed_triggered(self, i=None):
    if i==None: return
    # Start an immediate update for the current feed
    item=self.model.itemFromIndex(self.ui.feeds.currentIndex())
    if item and item.feed:
      # FIXME: move to out-of-process
      feedStatusQueue.put([0, item.feed.id])
      item.feed.update()
      feedStatusQueue.put([1, item.feed.id])
      self.open_feed(self.ui.feeds.currentIndex())

  def on_actionFetch_All_Feeds_triggered(self, i=None):
    if i==None: return
    global processes
    # Start an immediate update for all feeds
    statusQueue.put("fetching all feeds")
    p = processing.Process(target=feedUpdater, args=(True, ))
    p.setDaemon(True)
    p.start()
    processes.append(p)
    
  def on_actionAbort_Fetches_triggered(self, i=None):
    if i==None: return
    global processes, statusQueue, feedStatusQueue
    statusQueue.put("Aborting all fetches")
    # stop all processes and restart the background timed fetcher
    for proc in processes:
      proc.terminate()
    processes=[]
    
    # Mark all feeds as not updating
    for id in self.feedItems:
      feedStatusQueue.put([1, id])
    self.updateFeedStatus()
    self.updatesCounter=0
  
    p = processing.Process(target=feedUpdater)
    p.setDaemon(True)
    p.start()
    processes.append(p)
    
  def on_actionNext_Unread_Article_triggered(self, i=None):
    if i==None: return
    info( "Next Unread Article")
    if not self.currentFeed:
      self.on_actionNext_Unread_Feed_triggered(True)
    if self.currentPost:
      post=self.currentPost
    elif self.posts: 
      post=self.posts[0]
    else: # No posts in this feed, just go the next unread feed
      self.on_actionNext_Unread_Feed_triggered(True)
    if post.unread: # Quirk, should redo the flow
      nextPost=post
    else:
      nextPost=self.currentFeed.nextUnreadPost(post, self.ui.posts.model().sortOrder(), self.statusFilter)
    if nextPost:
      nextIndex=self.ui.posts.model().indexFromItem(self.postItems[nextPost.id])
      self.ui.posts.setCurrentIndex(nextIndex)
      self.on_posts_clicked(index=nextIndex)
    else:
      # At the end of the feed, go to next unread feed
      self.on_actionNext_Unread_Feed_triggered(True)
      
  def on_actionNext_Article_triggered(self, i=None, do_open=True):
    if i==None: return
    info ("Next Article")
    if self.currentPost:
      nextPost=self.currentFeed.nextPost(self.currentPost, self.ui.posts.model().sortOrder(), self.statusFilter)
    elif len(self.posts):
      nextPost=self.posts[0]
    else: # No posts in this feed, just go the next unread feed
      self.on_actionNext_Feed_triggered(True)
      return
    if nextPost:
      nextIndex=self.ui.posts.model().indexFromItem(self.postItems[nextPost.id])
      self.ui.posts.setCurrentIndex(nextIndex)
      self.on_posts_clicked(index=nextIndex)
    else:
      # At the end of the feed, go to next unread feed
      self.on_actionNext_Feed_triggered(True)

  def on_actionPrevious_Unread_Article_triggered(self, i=None):
    if i==None: return
    info("Previous Unread Article")
    if self.currentPost:
      post=self.currentPost
      previousPost=self.currentFeed.previousUnreadPost(post, self.ui.posts.model().sortOrder(), self.statusFilter)
    elif self.posts: # Not on a specific post, go to the last unread article
      previousPost=self.posts[-1]
      if not previousPost.unread:
        previousPost=self.currentFeed.previousUnreadPost(previousPost, self.ui.posts.model().sortOrder(), self.statusFilter)
    else:
      previousPost=None
    if previousPost:
      nextIndex=self.ui.posts.model().indexFromItem(self.postItems[previousPost.id])
      self.ui.posts.setCurrentIndex(nextIndex)
      self.on_posts_clicked(index=nextIndex)
    else:
      # At the beginning of the feed, go to previous feed
      self.on_actionPrevious_Unread_Feed_triggered(True)

  def on_actionPrevious_Article_triggered(self, i=None, do_open=True):
    if i==None: return
    info ("Previous Article")
    if self.currentPost:
      post=self.currentPost
      previousPost=self.currentFeed.previousPost(post, self.ui.posts.model().sortOrder(), self.statusFilter)
    elif self.posts: # Not on a specific post, go to the last article
      previousPost=posts[-1]
    else:
      previousPost=None
    if previousPost:
      nextIndex=self.ui.posts.model().indexFromItem(self.postItems[previousPost.id])
      self.ui.posts.setCurrentIndex(nextIndex)
      self.on_posts_clicked(index=nextIndex)
    else:
      # At the beginning of the feed, go to previous feed
      self.on_actionPrevious_Feed_triggered(True)

  def on_actionNext_Unread_Feed_triggered(self, i=None):
    if i==None: return
    info("Next unread feed")
    if self.currentFeed:
      nextFeed=self.currentFeed.nextUnreadFeed()
    else:
      nextFeed=root_feed.nextUnreadFeed()
    if nextFeed:
      self.open_feed(self.ui.feeds.model().indexFromItem(self.feedItems[nextFeed.id]))

  def on_actionNext_Feed_triggered(self, i=None):
    if i==None: return
    info("Next Feed")
    if self.currentFeed:
      nextFeed=self.currentFeed.nextFeed()
    else:
      nextFeed=root_feed.nextFeed()
    if nextFeed:
      self.open_feed(self.ui.feeds.model().indexFromItem(self.feedItems[nextFeed.id]))

  def on_actionPrevious_Feed_triggered(self, i=None):
    if i==None: return
    info("Previous Feed")
    if self.currentFeed:
      prevFeed=self.currentFeed.previousFeed()
      if prevFeed and prevFeed<>root_feed: # The root feed has no UI
        self.open_feed(self.ui.feeds.model().indexFromItem(self.feedItems[prevFeed.id]))
    # No current feed, so what's the meaning of "previous feed"?


  def on_actionPrevious_Unread_Feed_triggered(self, i=None):
    if i==None: return
    if self.currentFeed:
      prevFeed=self.currentFeed.previousUnreadFeed()
      if prevFeed and prevFeed<>root_feed: # The root feed has no UI
        self.open_feed(self.ui.feeds.model().indexFromItem(self.feedItems[prevFeed.id]))
    # No current feed, so what's the meaning of "previous unread feed"?
      
  def on_actionIncrease_Font_Sizes_triggered(self, i=None):
    if i==None: return
    self.ui.view.setTextSizeMultiplier(self.ui.view.textSizeMultiplier()+.2)
    
  def on_actionDecrease_Font_Sizes_triggered(self, i=None):
    if i==None: return
    self.ui.view.setTextSizeMultiplier(self.ui.view.textSizeMultiplier()-.2)

class FeedDelegate(QtGui.QItemDelegate):
  def __init__(self, parent=None):
    info("Creating FeedDelegate")
    QtGui.QItemDelegate.__init__(self, parent)
    
class PostDelegate(QtGui.QItemDelegate):
  def __init__(self, parent=None):
    info("Creating PostDelegate")
    QtGui.QItemDelegate.__init__(self, parent)
  
def exportOPML(fname):
  from OPML import Outline, OPML
  from cgi import escape
  def exportSubTree(parent, node):
    if not node.children:
      return
    for feed in node.children:
      co=Outline()
      co['text']=feed.text or ''
      if feed.xmlUrl:
        co['type']='rss'
        co['xmlUrl']=escape(feed.xmlUrl) or ''
        co['htmlUrl']=escape(feed.htmlUrl) or ''
        co['title']=escape(feed.title) or ''
        co['description']=escape(feed.description) or ''
      parent.add_child(co)
      
  opml=OPML()
  root=Outline()
  for feed in root_feed.children:
    exportSubTree(root, feed)
  opml.outlines=root._children
  opml.output(open(fname, 'w'))
    
def importOPML(fname, parent=root_feed):

  def importSubTree(parent, node):
    if node.tag<>'outline':
      return # Don't handle
    xu=node.get('xmlUrl')
    if xu:
      # If it's already somewhere, don't duplicate
      f=Feed.get_by(xmlUrl=node.get('xmlUrl'), 
             htmlUrl=node.get('htmlUrl'), 
             )
      if not f:
        f=Feed(xmlUrl=node.get('xmlUrl'), 
             htmlUrl=node.get('htmlUrl'), 
             title=node.get('title'),
             text=node.get('text'), 
             description=node.get('description'), 
             parent=parent
             )
        # Add any missing stuff (mainly icons, I guess)
        # Kills performance, though
        # f.updateFeedData()
    else: # Let's guess it's a folder
      f=Feed.get_by_or_init(text=node.get('text'), parent=parent)
      for child in node.getchildren():
        importSubTree(f, child)

  from xml.etree import ElementTree
  tree = ElementTree.parse(fname)
  for node in tree.find('//body').getchildren():
    importSubTree(parent, node)
  session.flush()

# The feed updater (runs out-of-process)
def feedUpdater(full=False):
  initDB()
  if full:
      for feed in Feed.query.filter(Feed.xmlUrl<>None):
        feedStatusQueue.put([0, feed.id])
        try: # we can't let this fail or it will stay marked forever;-)
          feed.update()
        except:
          pass
        feedStatusQueue.put([1, feed.id])
  else:
    while True:
      info("updater loop")
      time.sleep(60)
      now=datetime.now()
      for feed in Feed.query.filter(Feed.xmlUrl<>None):
        period=1800 # FIXME: make this configurable
        if feed.updateInterval==0: # Update never
          continue
        elif feed.updateInterval<>-1: # not update default
          period=60*feed.updateInterval # convert to seconds
        if (now-feed.lastUpdated).seconds>period:
          info("updating because of timeout")
          feedStatusQueue.put([0, feed.id])
          try: # we can't let this fail or it will stay marked forever;-)
            feed.update()
          except:
            pass
          feedStatusQueue.put([1, feed.id])

from BeautifulSoup import BeautifulStoneSoup 
def decodeString(s):
  '''Decode HTML strings so you don't get &lt; and all those things.'''
  u=unicode(BeautifulStoneSoup(s,convertEntities=BeautifulStoneSoup.HTML_ENTITIES ))
  return u

def main():
  global root_feed
  initDB()
    
  if len(sys.argv)>1:
      # Import a OPML file into the DB so we have some data to work with
      importOPML(sys.argv[1], root_feed)
  app=QtGui.QApplication(sys.argv)
  window=MainWindow()
  
  # This will start the background fetcher as a side effect
  window.on_actionAbort_Fetches_triggered(True)
  window.show()
  sys.exit(app.exec_())
  
if __name__ == "__main__":
  main()



"""easylog wraps the logging module, to reduce the required glue.

The standard logging module is very useful, but it does not easily
scale *down* for simple uses.  easylog provides the required
configuration and glue code, so that scripts and application modules
can treat logging as a one-liner,

Sample Lightweight Usage
========================

    >>> from easylog import critical, error, warning, debug, info
    >>> debug("getFoo requesting %d %s", 42, "answers")
    >>> error("getFoo says %d of those %s are lousy!", 19, "answers")
    ERROR:easylog.easylog:getFoo says 19 of those answers are lousy!

The debug message is captured in (current directory) file EasyLogA,
but is not echoed to the screen.  The error message goes both places.


Sample Midweight Usage
======================

If you want slightly more control, you can use more of the easylog
API.  Additional examples can be found in the default setup at the
bottom of this file.

    >>> import easylog
    >>> from easylog import debug, info, warn, error, fatal
    
    >>> # log as MyApp, regardless of the current filename.
    ... # ignore debug messages.
    ... easylog.setLogger(name='MyApp', level=easylog.INFO)
    <easylog.easyLogger instance at 0x813207c>
    
    >>> # messages from "mytrace" are really from its caller.
    ... easylog.ignoreCaller("mytrace")

    >>> warn("Debug messages will be sent but ignored.")

The default configuration sends all messages to a rotating logfile.
(EasyLogA in the current directory)

The default configuration also echoes important messages to the
standard error stream.  fatal or critical messages are always
important.  Errors are important, but are filtered to prevent
repeats of the exact same message.  Warnings are important, but
are not repeated even if the parameters (such as which file is
missing) change.


Specialized Usage
=================

If you want even more control, then importing * should expose
(a superset of) all functionality available through the underlying
logging package, with the following restrictions:

    (1)  All easylog calls other than "disable" are rooted
         on (virtual root) logging.root.easylog,rather than
         directly on logging.root.

         This reduces (but does not eliminate) interference
         from other processes using the same logging system.

    (2)  Direct manipulation of the logging tree is not supported.

By using easylog instead of using the underlying logging package
directly, you will still gain these advantages.

All paramaters which can be set through either a logging config file
or set* functions can be set through keyword arguments to the initial
class constructor.  This is true even when the underlying library does
not support configuration at construction time.

Most parameters that can be set at initial construction can be changed
later through a set* function.  This is sometimes true even when the
underlying library does not support reconfiguration.


Naming conventions
==================

The logging package uses:

    MixedCase for classes
    camelCase for functions and methods
    UPPER_CASE for global variables.

easylog follows this convention, but does expose global functions
under the lower_case alias to support the recommended python style.
Class methods are not usually given aliases; you will still need to
use camelCase.


Notable extensions to the logging API
=====================================

addLevelName (level, levelname)
-------------------------------

add_level_name(5, "DEBUG_VERBOSE") creates a DEBUG_VERBOSE constant
for the log function and for display or use with filter functions.
The easylog version also creates a debug_verbose convenience function.

add_level_name is an alias.


ignoreCaller(filename)
-----------------

Treat name as part of the logging system.  Functions in this file/module
are ignored when determining where a logging message originated.

ignore_caller is an alias.


setLogger(name=None, level=None, handlers=None, propagate=None)
===============================================================

This overrides logging getLogger, but the set name is preferred,
as several options can be changed.  If an argument is None or
is not passed, the previous value will be kept.  If this is a
new logger, the default values will be used.

getLogger, set_logger, and get_logger are all aliases.


FilterUnique(minLevel=None, maxLevel=None, exact=False, hide=None)
==================================================================

Creates a filter to suppress duplicate messages.

Messages of importance less than minLevel or greater than maxLevel
are ignored; they will be filtered or approved by other functions.

If exact is True, then messages will only be suppresed if they match
exactly *after* string interpolation of any additional arguments.
If exact is False, the messages will be considered duplicate even if
the values of the parameters change::

    error("Bad Value: %d", 5)
    error("Bad Value: %d", -43)

are the same if exact is False, but different if exact is True.

hide is a dictionary of already-seen messages.  The default is to
create a new empty dictionary when the filter is constructed.

minLevel and maxLevel can also be spelled min_level and max_level.

If there is no max_level, the default is equal to the min_level.


DefaultConsole
==============

Important messages are sent to Standard Error.  Important means
unique WARNINGs, exactly unique ERRORs, and any CRITICAL or FATAL errors.


DefaultLogFile
==============

All messages are logged here, unless a user has in some way explicitly
suppressed them.


"""

import sys
import logging
import types
from os.path import split, splitext, basename, normcase

try:
    from atexit import register as register_exit
except ImportError:
    def register_exit(fn):
        oldexit = sys.exit
        def exithook(*args, **kwargs):
            fn()
            oldexit(*args, **kwargs)
        sys.exit = exithook
    

# __all__ will be extended by names exported from the underlying logging
# and logging.handlers packages, including convenience functions,
# handlers and level-names
__all__ = []


__copyright__ = """Copyright 2004, Jim J. Jewett.  All rights reserved.

easylog is licensed under the Python Software Foundation license.

Please send any comments or patches to JimJJewett@yahoo.com, or see
<URL: http://sourceforge.net/projects/easylog >

"""

__version__ = (0, 2, 0, "alpha1", 0)

loggers = {}

# grep for lines starting with 'levelname 20' to find when they were logged
# grep for lines starting with 'levelname fr' to find the source file/line
# grep for lines starting with 'levelname: ' to find the actual messages.
#
# If filenames and messages are reasonably short, then the log will fit
#    in 80 characters without linewrap.
EXTENDED_FORMAT = ("%(levelname)s %(asctime)s " +
                      "in thread %(process)d.%(thread)d\n" +
                   "%(levelname)s from %(name)s file %(filename)s " +
                      "line %(lineno)d\n" +
                   "%(levelname)s: %(message)s\n")

########################################################################
#
# For compatibility with python versions < 2.3
#
########################################################################
try:
    (True, False)
except NameError:
    # match the 2.2 definition
    (True, False) = (1, 0)

def dictpop(d, k, default=None):
    """dict.pop() is new in 2.3"""
    try:
        ret = d.pop(k, default)
    except AttributeError:
        try:
            ret = d[k]
            del d[k]
        except KeyError:
            ret = default
    return ret


########################################################################
#
# code templates, used for statements which depend on the contents
# of other modules.  Use of templates provides some small degree
# of protection against changes in the underlying library.
#
########################################################################

convenience_fn_string = (
"""
def %(levelLow)s(*args, **kwargs):
    chooseLogger().%(levelLow)s(*args, **kwargs)
    
exec("globals()['%(levelLow)s'] = %(levelLow)s")    

"""
)

handler_mixin_string = (
"""
class %(name)s(%(parentClass)s):
    '''%(name)s augments %(parentClass)s with set methods and init keywords.'''
    
    # Note that we cannot use super(thisclass, self) because logging.Handler 
    # does not inherit from object.  Even if we add object as a base, we are
    # just asking for trouble if our own parent classes are not designed as
    # new style classes.

    # class-wide defaults
    msg_fmt = None
    time_fmt = None
    formatclass = logging.Formatter

    def __init__(self, *args, **kwargs):
        '''logging message handler %(name)s

        level is the minimum level that is not ignored.
        formatter is the formatting function.
        fmt or format is the format string used by formatter.
        datefmt or dateformat is the date/time formatting string.
        If any element of filters returns False, the message is suppressed.
        
        '''
        # Using kwargs directly fails when we split out the formatting
        # portion, as **kwargs passes a copy, so that the keywords are
        # not removed from the original kwargs -- and they are still
        # there to the detriment of the parent class.
        self._ini = kwargs
        level = dictpop(self._ini, 'level', None)
        filters = dictpop(self._ini, 'filters', [])

        # Call this before parent.__init__ to remove the keywords.
        # But the parent may explicitly set formatter to None, and then
        # use its own default ... so we have to save the result.
        self.setFormat()
        __tempfmt = self.formatter

        # Note that we pass through any unrecognized arguments; the
        # standard logging module will raise an exception if it does 
        # not understand them either.
        %(parentClass)s.__init__(self, *args, **self._ini)

        self.formatter = __tempfmt
        if level:
            self.setLevel(level)
        for f in filters:
            self.addFilter(f)

    def setFormat(self, **kwargs):
        '''If format or dateformat was passed, use it.

        Otherwise use the current value.  If there is no current value,
        then fall through to the default of the parent class.  Note that
        this assumes the Formatter API, though it does go through
        contortions to work with BufferingFormatter or a Formatter which
        takes no parameters.

        '''
        self._ini.update(kwargs)
        self.formatclass = dictpop(kwargs, 'formatter',
                                   self.formatclass)
        self.msg_fmt = (dictpop(self._ini, 'fmt', None) or
                        dictpop(self._ini, 'format', None) or
                        dictpop(self._ini, 'linefmt',
                                self.msg_fmt))
        self.time_fmt = (dictpop(self._ini, 'datefmt', None) or
                         dictpop(self._ini, 'dateformat',
                                 self.time_fmt))

        # Do not pass an explicit None, as BufferingFormatter chokes on
        # the second (date format) argument.
        if self.time_fmt:
            self.setFormatter(self.formatclass(self.msg_fmt, self.time_fmt))
        elif self.msg_fmt:
            self.setFormatter(self.formatclass(self.msg_fmt))
        else:
            self.setFormatter(self.formatclass())            

    def close(self):
        '''ensure flush() before close.  Call me paranoid.'''
        try:
            self.flush()
        except ValueError:
            pass
        %(parentClass)s.close(self)
"""
)

########################################################################
#
# Specific overrides of module-level symbols.
#
########################################################################

# The easylog classes do not inherit from this improved Filterer, but
# we make it available so that our own callers can use the "create it
# right in the first place" API that we export for other classes.
# Note that the only recognized argument is filters -- anything else
# will be passed through and therefore will create an exception.
class Filterer(logging.Filterer):
    def __init__(self, filters=None, *args, **kwargs):
        logging.Filterer.__init__(self, *args, **kwargs)
        if filters:
            for filter in filters:
                self.addFilter(filter)

# In addition to calling logging.addLevelName, we also provide the
# expected convenience functions.
# By convention, levelnames are all caps, and convenience functions
# are all lower case.  An all-lower-case level-name will be shadowed
# by its convenience function.
def addLevelName(level, levelName, knownname=False):
    if not knownname:
        logging.addLevelName(level, levelName)
    varset = {'levelName': levelName,
              'levelLow' : levelName.lower(),
              'levelno': level}
    exec ("globals()['%(levelName)s'] = %(levelno)d" % varset)
    exec(convenience_fn_string % varset)
    exec("globals()['%(levelLow)s'] = %(levelLow)s" % varset)
    __all__.extend((levelName,
                    levelName.lower()))

add_level_name = addLevelName

def shutdown():
    """Safely shut down logging.

    Earlier versions of logging (including that distributed with
    python 2.3) required that the user call logging.shutdown()
    exactly once.  The current library is safer, but this will
    work regardless of version.
    
    The guard clause protects against someone else calling it first.
    The clear protects against repeated calls.

    """
    if logging._handlers:
        logging.shutdown()
        logging._handlers.clear()

register_exit(shutdown)

########################################################################
#
# imports of module level sybmols
#
########################################################################
aliases = ['setLogger', 'set_logger', 'get_logger',
           'add_level_name', 'get_level_name',
           'make_log_record',
           'ignoreCaller']

newdefs = ['EXTENDED_FORMAT',
           'ignore_caller',
           'FilterUnique',
           'DefaultConsole',
           'DefaultLogFile']
new_convenience_functions = ['addHandler', 'removeHandler', 'setHandlers']
nonlevel_convenience_functions = ['exception', 'log']
overridden = ['getLogger',
              'addLevelName',
              'shutdown',
              'Filterer']
passthrough = ['makeLogRecord', 'LogRecord',
               'getLevelName',
               'BASIC_FORMAT',
               'disable']
skipped = ['Logger', 'Manager', 'PlaceHolder', 'setLoggerClass',
           'RootLogger', 'root',
           'Filter',
           'NOTSET', 'raiseExceptions', 
           'basicConfig']

redefined = (newdefs +
             new_convenience_functions +
             nonlevel_convenience_functions +
             overridden)

# These exist, because easylog defines them.
# names in passthrough *should* exist, but we'll wait until we're sure.
__all__.extend(redefined + aliases)

ignored = redefined + skipped 

leftovers=[]
phantoms=[]

# import most of logging (levels, convenience functions)
for (k, v) in vars(logging).items():
    # These names should not exist in the underlying logging package.
    if k in aliases:
        phantoms.append(k)
        continue
    # These we either replace or do not expose
    if k in ignored:
        continue
    if k in passthrough:
        exec ("from logging import %s" % k)
        __all__.append(k)
        continue

    # Do not export private or internal variables
    if k.startswith('_'):
        continue
    # Do not export other modules
    if isinstance(v, types.ModuleType):
        continue

    # Should we add set methods?  Maybe not useful.  Also note
    # that Formatter and BufferingFormatter already have different
    # API and internal variables; the simplicity would be synthetic.
    # Within a handler, we change Formatter options by creating an
    # entirely new object.
    if k.endswith('Formatter'):
        exec("from logging import %s" % k)
        __all__.append(k)
        continue

    # FileHandler, StreamHandler, and the abstract base class Handler
    if k.endswith('Handler'):
        exec(handler_mixin_string %
             {'name': k, 'parentClass': ("logging.%s" % k) })
        __all__.append(k)
        continue
    
    # These should be levelnames, such as debug/DEBUG or warn/WARN.
    # We replace these functions/variables.
    if k == k.lower() and k.upper() in vars(logging):
        continue
    
    # All levelnames (so far) are in all_caps
    if k in logging._levelNames:
        addLevelName(v, k, True)
        continue

    # special case -- logging forgot to include the alias in _levelnames,
    # but if we add it, then it becomes the primary name.  So we add it
    # another way.  When the underlying package is new enough to correct
    # this, then it will be caught by the previous case, and this
    # branch will become dead code.
    if k == 'FATAL':
        addLevelName(v, k, True)
        logging._levelNames[k] = v
        continue
        
    # If the code gets here, then the logging module is newer (or at
    # least different) than expected; log a debug message later.
    leftovers.append((k,v))
    
for newfn in new_convenience_functions:
    exec(convenience_fn_string % {'levelLow': newfn})
    
try:
    import logging.handlers
    handler_list = [handler
                    for handler in vars(logging.handlers)
                    if handler.endswith('Handler')]

    for k in handler_list:
        exec(handler_mixin_string %
             {'name': k, 'parentClass': ("logging.handlers.%s" % k) })

    __all__.extend(handler_list)
    
except ImportError:
    pass

########################################################################
#
#  symbols added by the easylog library.
#
########################################################################

get_level_name = getLevelName
make_log_record = makeLogRecord

def name_as_module (name):
    name = basename(name)
    name = splitext(name)[0]
    return name
    
# These "callers" are really part of the logging system.
# Get *their* callers.
fake_callers = ["<stdin>", "<string>", 'easylog']

def ignore_caller(name):
    """Add a __file__ to the ignore list."""
    if name not in fake_callers:
        fake_callers.append(name)
        shortname = name_as_module (name)
        if shortname not in fake_callers:
            fake_callers.append(name)

ignoreCaller = ignore_caller

try:
    ignore_caller(__file__)
except NameError:
    pass
try:
    ignore_caller(logging.__file__)
except AttributeError:
    pass
try:
    ignore_caller(logging._srcfile)
except AttributeError:
    pass
try:
    ignore_caller(logging.handlers.__file__)
except AttributeError:
    pass


def fake_caller(name):
    """Is this "caller" really part of the logging system?"""
    if name in fake_callers or name_as_module(name) in fake_callers:
        return True
    
    (path, name) = split(name)
    if normcase(path).endswith('logging'):
        return True

    return False

unknown_file_name = "<unknown file>"
def findLogCaller():
    """Return source filename (and line number) of the caller.
    
    Based on logging.Logger().findCaller(), but excludes easylog as well.
    
    """
    try:
        f = sys._getframe(1)
        while True:
            co = f.f_code
            filename = co.co_filename
            if fake_caller(filename):
                f = f.f_back
                continue
            return filename, f.f_lineno
    # The logging module suggests that _getframe may not exist (despite
    # the documentation).  We could also reach the top frame (so that
    # there is no f_back) and still be in excluded files.
    except (AttributeError, ValueError):
        return unknown_file_name, 0

def chooseLogger(name=None):
    """Which logger should this caller use?"""

    # Who called?
    if name is None:
        name, ignored = findLogCaller()

    # Normalize the name.
    if name == unknown_file_name:
        name = 'easylog'
    else:
        name = "easylog." + name_as_module(name)

    # Does the caller already have a logger?  Use our dict
    # first, so that a multi-file app can easily send all
    # output to a single logger by calling
    # setLogger(name="myapp") in each file.
    try:
        logger = loggers[name]
    except KeyError:
        logger = logging.getLogger(name)
        loggers[name] = logger
    return logger

choose_logger = chooseLogger

class easyLogger(logging.Logger):

    def callHandlers(self, record, **kwargs):
        """
        Pass a record to all relevant handlers.

        Loop through all handlers for this logger and its parents in the
        logger hierarchy. If no handler was found, output a one-off error
        message to sys.stderr. Stop searching up the hierarchy whenever a
        logger with the "propagate" attribute set to zero is found - that
        will be the last logger whose handlers are called.

        Modified to pass through kwargs
        Modified to count and return actual emits.  (in place of None)

        """
        c = self
        used = found = 0
        while c:
            for hdlr in c.handlers:
                found = found + 1
                if record.levelno >= hdlr.level:
                    used += hdlr.handle(record, **kwargs)
            if not c.propagate:
                c = None    #break out
            else:
                c = c.parent

        # No handlers is a warning, but without a handler, we cannot really
        # emit it, nor can we use filterUnique to suppress the next such
        # call -- so we count emits in an attribute on the manager. 
        if not found and not self.manager.emittedNoHandlerWarning:
            sys.stderr.write("No handlers could be found for logger"
                             " \"%s\"\n" % self.name)
            self.manager.emittedNoHandlerWarning = 1
            
        return used

    def exception(self, msg, *args, **kwargs):
        """
        Convenience method for logging an ERROR with exception information.

        Modified to pass through additional keywords.

        """
        if 'exc_info' not in kwargs:
            kwargs['exc_info'] = True
        self.error(msg, *args, **kwargs)

    def findCaller(self):
        """
        Log a message from where?

        Ensure that easylog is not mistaken for the caller.

        Find the stack frame of the caller so that we can note the source
        file name and line number.
        
        """
        return findLogCaller()

    def getHandlers(self):
        return self.handlers
    
    def handle(self, record, **kwargs):
        """
        Call the handlers for the specified record.

        This method is used for unpickled records received from a socket, as
        well as those created locally. Logger-level filtering is applied.

        Overridden to pass through kwargs.

        """
        if (not self.disabled) and self.filter(record):
            self.callHandlers(record, **kwargs)

    def _log(self, level, msg, args, exc_info=None, **kwargs):
        """
        Low-level logging routine which creates a LogRecord and then calls
        all the handlers of this logger to handle the record.

        Override to realize that easylog is not the actual caller, and
        to pass keyword arguments through.

        """
        fn, lno = self.findCaller()
        if exc_info and type(exc_info) != types.TupleType:
            exc_info = sys.exc_info()
        record = self.makeRecord(self.name, level,
                                 fn, lno,
                                 msg, args, exc_info)
        self.handle(record, **kwargs)

    def removeHandler(self, hdlr):
        """
        Remove the specified handler from this logger.

        This differs from base class in that it flushes first.
        It does not close, as base class made an explicit decision not to,
        in case the same handler is used by multiple loggers.  (Note that
        the duplicate usage may not even be within the easylog framework.)
        
        """
        if hdlr in self.handlers:
            hdlr.flush()
            self.handlers.remove(hdlr)

    def setHandlers(self, newHandlers=None, propagate=None,
                    new_handlers={}):
        """Set the handlers to (only) of newHandlers.

        propagate=True indicates that more general loggers should still
        be allowed to see the messages.  False indicates that this is a
        complete set, and None indicates no change.

        new_handlers is an alternative spelling of newHandlers.

        Add new handlers before removing old ones in case another thread
        emits log records in the middle of this operation.

        """
        if newHandlers is None:
            newHanders = new_handlers
        for newHandler in newHandlers:
            self.addHandler(newHandler)
        for oldHandler in self.handlers:
            if oldHandler not in newHandlers:
                self.removeHandler(oldHandler)
        if propagate is not None:
            self.propagate = propagate


logging.setLoggerClass(easyLogger)
    
def set_logger(name=None, level=None, handlers=None, propagate=None):
    """Set properties for the requested logger (default to the calling file).

    If the logger does not already exist, it will be created.  If it does,
    the attribute values will be changed as requested.  None leaves a value
    unchanged.  In particular, use propagate=False to shut off propagation.

    """

    logger = choose_logger(name)
    if name is not None:
        # If the caller specified a logger name, it applies to the whole
        # file (at least until they change it again).  This saves the user
        # from needing to keep a reference and use bound methods.
        f, ignored = findLogCaller()
        if f == unknown_file_name:
            f = "easylog"
        loggers[f] = logger

    # level = 0 is valid, but beware:  The logging package treats it specially.
    if level is not None:
        logger.setLevel(level)

    if handlers is not None:
        logger.setHandlers(handlers)

    if propagate is not None:
        logger.propagate = propagate
    elif not logger.getHandlers():
        # If there are no handlers here, propagate by default.
        logger.propagate = True

    logger.easylog_setup = True

    # The user does not need to keep any references, but they do have
    # the option, in case they want to alternate loggers for some
    # reason.
    return logger

# Maintain an API consistent with logging, but also with python
# style guides.
getLogger = setLogger = get_logger = set_logger

########################################################################
#
# Nothing is actually inherited from logging.Filter, and the only API
# is a method named filter which takes a record and returns True if
# the record is *not* thrown out by this filter.  (Think of it as "keepif"
# instead of "filter", but any filter may veto.)  We inherit in case
# someone has written code using isinstance.
#
########################################################################
class FilterUnique(logging.Filter):

    """Ensures that a message will only appear once.

    exact=False means to ignore parameters, such as *which* file could
    not be read.

    If minLevel or maxLevel are set, then messages outside that range
    will be ignored.  They may be repeated, and they will not prevent
    the appearance of the first message with an appropriate level.
    (min_level and max_level are alternative spellings, to support the
    python style guide, rather than the logging style.)

    hide is the dictionary of initially hidden messages.

    min_level and max_level are alternative spellings for minLevel
    and maxLevel, respectively.

    """
    
    def __init__(self,
                 minLevel = None, maxLevel=None,
                 exact=False,
                 hide=None,
                 min_level=None, max_level=None):
        self.min_level = minLevel or min_level
        self.max_level = maxLevel or max_level or self.min_level
        self.exact = exact
        self.hide = hide or {}

    def filter(self, record):
        "Return True unless this filter vetoes the record."""

        # Return True (OK) if this filter is not applicable.
        if self.min_level is not None and self.min_level > record.levelno:
            return True
        if self.max_level is not None and self.max_level < record.levelno:
            return True
        
        # Supress "duplicate" messages.
        if self.exact:
            key = record.getMessage()
        else:
            key = record.msg
        # Already seen?
        if key in self.hide:
            return False
        # This is the first time.  Make sure it is also the last time.
        # (Well, maybe unless it comes through with a different priority.)
        self.hide[key] = True
        return True
        

##########################################################################
#                                                                        #
# Examples (and default setup)                                           #
#                                                                        #
# Below here is sample complex usage, used for the defaults.             #
#                                                                        #
##########################################################################

class DefaultConsole(StreamHandler):
    """send Warnings on up to stderr.

    Supress messages less important than a Warning.

    Each Warning is sent only once.

    Each Error is sent once per detailed message.  (Example:  Failing to
    open a file would display only once, but failing to open a different file
    would still be reported.)

    Critical messages are always sent.

    """
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('level', WARNING)
        kwargs.setdefault('filters', (FilterUnique(WARNING),
                                      FilterUnique(ERROR, ERROR, exact=True)))
        kwargs.setdefault('format', BASIC_FORMAT)
        
        StreamHandler.__init__(self, *args, **kwargs)


class DefaultLogFile(RotatingFileHandler):
    """Log everything to (current directory).EasyLogA.*

    Rollover at the start of each run.
    Only keep 100K/file, to avoid resource exhaustion.
    Normally keep 7 runs ~= a week's worth.

    """
    def __init__(self, *args, **kwargs):

        # Defaults to logging in current working directory,
        kwargs.setdefault('filename', "EasyLogA")

        # An infinite loop can fill maxBytes * backupCount, so make it small
        kwargs.setdefault('maxBytes', 100000)
        kwargs.setdefault('backupCount', 7)

        kwargs.setdefault('format', EXTENDED_FORMAT)

        # 0 (==logging.NOTSET) gets special-cased to really be WARNING,
        # so DEBUG and INFO messages are discarded.
        # If the user has not specified a level, then we should log
        # everything they say to log, at least to the logfile.
        kwargs.setdefault('level', -1)
        
        RotatingFileHandler.__init__(self, *args, **kwargs)

        # Start a new logfile with each session.
        self.doRollover()


setLogger(name=None, level=-1, propagate=False,
          handlers=(DefaultConsole(), DefaultLogFile()))

# We use getLogger() because calls from within this file are not
# recognized as the "real" caller.
internal = getLogger('easylog')

internal.info("Logging Started (easylog %d.%d.%d %s patch %d)" %  __version__)

if leftovers:
    internal.debug("Unexpected symbols from logging package: %s", leftovers)
    
if phantoms:
    internal.warning("Overriding unexpected symbols from logging package: %s",
                     phantoms)
    



'''An attempt at a setup.py that can be used with py2exe on windows. It's
not even close to working, though'''

from py2exe.build_exe import py2exe
from distutils.core import setup


setup(windows=['urssus.py'],
      options={
                "py2exe":{
                        "includes": ["elixir","sip"],                        
                }
        }
)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'urssus/ui/filterwidget.ui'
#
# Created: Sun Jul 27 21:58:01 2008
#      by: PyQt4 UI code generator 4.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(559,28)
        self.horizontalLayout = QtGui.QHBoxLayout(Form)
        self.horizontalLayout.setMargin(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtGui.QLabel(Form)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.filter = QtGui.QLineEdit(Form)
        self.filter.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.filter.setObjectName("filter")
        self.horizontalLayout.addWidget(self.filter)
        self.clear = QtGui.QToolButton(Form)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/editclear.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.clear.setIcon(icon)
        self.clear.setObjectName("clear")
        self.horizontalLayout.addWidget(self.clear)
        self.label_2 = QtGui.QLabel(Form)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.statusCombo = QtGui.QComboBox(Form)
        self.statusCombo.setObjectName("statusCombo")
        self.horizontalLayout.addWidget(self.statusCombo)
        self.label.setBuddy(self.filter)
        self.label_2.setBuddy(self.statusCombo)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtGui.QApplication.translate("Form", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Form", "Sear&ch:", None, QtGui.QApplication.UnicodeUTF8))
        self.clear.setText(QtGui.QApplication.translate("Form", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Form", "Status:", None, QtGui.QApplication.UnicodeUTF8))
        self.statusCombo.addItem(QtGui.QApplication.translate("Form", "All Articles", None, QtGui.QApplication.UnicodeUTF8))
        self.statusCombo.addItem(QtGui.QApplication.translate("Form", "Unread", None, QtGui.QApplication.UnicodeUTF8))
        self.statusCombo.addItem(QtGui.QApplication.translate("Form", "Important", None, QtGui.QApplication.UnicodeUTF8))

import icons_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'urssus/ui/main.ui'
#
# Created: Sun Jul 27 21:58:02 2008
#      by: PyQt4 UI code generator 4.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800,600)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/urssus.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setIconSize(QtCore.QSize(24,24))
        MainWindow.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        self.centralWidget = QtGui.QWidget(MainWindow)
        self.centralWidget.setGeometry(QtCore.QRect(0,70,800,506))
        self.centralWidget.setObjectName("centralWidget")
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.centralWidget)
        self.verticalLayout_2.setMargin(1)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter_2 = QtGui.QSplitter(self.centralWidget)
        self.splitter_2.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_2.setObjectName("splitter_2")
        self.feeds = QtGui.QTreeView(self.splitter_2)
        self.feeds.setFocusPolicy(QtCore.Qt.NoFocus)
        self.feeds.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.feeds.setFrameShape(QtGui.QFrame.Panel)
        self.feeds.setFrameShadow(QtGui.QFrame.Plain)
        self.feeds.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.feeds.setAlternatingRowColors(True)
        self.feeds.setTextElideMode(QtCore.Qt.ElideMiddle)
        self.feeds.setIndentation(20)
        self.feeds.setRootIsDecorated(True)
        self.feeds.setUniformRowHeights(True)
        self.feeds.setAnimated(True)
        self.feeds.setAllColumnsShowFocus(True)
        self.feeds.setHeaderHidden(True)
        self.feeds.setObjectName("feeds")
        self.splitter = QtGui.QSplitter(self.splitter_2)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName("splitter")
        self.posts = QtGui.QTreeView(self.splitter)
        self.posts.setFocusPolicy(QtCore.Qt.NoFocus)
        self.posts.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.posts.setFrameShape(QtGui.QFrame.Panel)
        self.posts.setFrameShadow(QtGui.QFrame.Plain)
        self.posts.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.posts.setAlternatingRowColors(True)
        self.posts.setRootIsDecorated(False)
        self.posts.setUniformRowHeights(True)
        self.posts.setItemsExpandable(False)
        self.posts.setSortingEnabled(True)
        self.posts.setAnimated(True)
        self.posts.setAllColumnsShowFocus(True)
        self.posts.setExpandsOnDoubleClick(False)
        self.posts.setObjectName("posts")
        self.frame = QtGui.QFrame(self.splitter)
        self.frame.setFrameShape(QtGui.QFrame.Panel)
        self.frame.setFrameShadow(QtGui.QFrame.Plain)
        self.frame.setObjectName("frame")
        self.verticalLayout = QtGui.QVBoxLayout(self.frame)
        self.verticalLayout.setMargin(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.view = QtWebKit.QWebView(self.frame)
        self.view.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.view.setUrl(QtCore.QUrl("about:blank"))
        self.view.setObjectName("view")
        self.verticalLayout.addWidget(self.view)
        self.verticalLayout_2.addWidget(self.splitter_2)
        MainWindow.setCentralWidget(self.centralWidget)
        self.toolBar = QtGui.QToolBar(MainWindow)
        self.toolBar.setGeometry(QtCore.QRect(0,31,155,39))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/urssus.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.toolBar.setWindowIcon(icon)
        self.toolBar.setIconSize(QtCore.QSize(24,24))
        self.toolBar.setObjectName("toolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea,self.toolBar)
        self.menuBar = QtGui.QMenuBar(MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0,0,800,31))
        self.menuBar.setObjectName("menuBar")
        self.menuFeed = QtGui.QMenu(self.menuBar)
        self.menuFeed.setObjectName("menuFeed")
        self.menu_File = QtGui.QMenu(self.menuBar)
        self.menu_File.setObjectName("menu_File")
        self.menu_Go = QtGui.QMenu(self.menuBar)
        self.menu_Go.setObjectName("menu_Go")
        self.menuHelp = QtGui.QMenu(self.menuBar)
        self.menuHelp.setObjectName("menuHelp")
        self.menu_View = QtGui.QMenu(self.menuBar)
        self.menu_View.setObjectName("menu_View")
        self.menu_View_Mode = QtGui.QMenu(self.menu_View)
        self.menu_View_Mode.setObjectName("menu_View_Mode")
        self.menu_Edit = QtGui.QMenu(self.menuBar)
        self.menu_Edit.setObjectName("menu_Edit")
        self.menuArticle = QtGui.QMenu(self.menuBar)
        self.menuArticle.setObjectName("menuArticle")
        MainWindow.setMenuBar(self.menuBar)
        self.statusBar = QtGui.QStatusBar(MainWindow)
        self.statusBar.setGeometry(QtCore.QRect(0,576,800,24))
        self.statusBar.setObjectName("statusBar")
        MainWindow.setStatusBar(self.statusBar)
        self.filterBar = QtGui.QToolBar(MainWindow)
        self.filterBar.setGeometry(QtCore.QRect(155,31,645,39))
        self.filterBar.setObjectName("filterBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea,self.filterBar)
        self.actionFetch_Feed = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/1downarrow.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionFetch_Feed.setIcon(icon)
        self.actionFetch_Feed.setObjectName("actionFetch_Feed")
        self.actionFetch_All_Feeds = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/2downarrow.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionFetch_All_Feeds.setIcon(icon)
        self.actionFetch_All_Feeds.setObjectName("actionFetch_All_Feeds")
        self.actionAbort_Fetches = QtGui.QAction(MainWindow)
        self.actionAbort_Fetches.setEnabled(False)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/stop.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionAbort_Fetches.setIcon(icon)
        self.actionAbort_Fetches.setObjectName("actionAbort_Fetches")
        self.actionMark_Feed_as_Read = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/apply.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionMark_Feed_as_Read.setIcon(icon)
        self.actionMark_Feed_as_Read.setObjectName("actionMark_Feed_as_Read")
        self.actionImport_Feeds = QtGui.QAction(MainWindow)
        self.actionImport_Feeds.setObjectName("actionImport_Feeds")
        self.actionExport_Feeds = QtGui.QAction(MainWindow)
        self.actionExport_Feeds.setObjectName("actionExport_Feeds")
        self.actionQuit = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/exit.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionQuit.setIcon(icon)
        self.actionQuit.setObjectName("actionQuit")
        self.actionNext_Article = QtGui.QAction(MainWindow)
        self.actionNext_Article.setObjectName("actionNext_Article")
        self.actionNext_Unread_Article = QtGui.QAction(MainWindow)
        self.actionNext_Unread_Article.setObjectName("actionNext_Unread_Article")
        self.actionNext_Feed = QtGui.QAction(MainWindow)
        self.actionNext_Feed.setObjectName("actionNext_Feed")
        self.actionNext_Unread_Feed = QtGui.QAction(MainWindow)
        self.actionNext_Unread_Feed.setObjectName("actionNext_Unread_Feed")
        self.actionAbout_uRSSus = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/urssus.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionAbout_uRSSus.setIcon(icon)
        self.actionAbout_uRSSus.setObjectName("actionAbout_uRSSus")
        self.actionIncrease_Font_Sizes = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/viewmag+.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionIncrease_Font_Sizes.setIcon(icon)
        self.actionIncrease_Font_Sizes.setObjectName("actionIncrease_Font_Sizes")
        self.actionDecrease_Font_Sizes = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/viewmag-.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionDecrease_Font_Sizes.setIcon(icon)
        self.actionDecrease_Font_Sizes.setObjectName("actionDecrease_Font_Sizes")
        self.actionStatus_Bar = QtGui.QAction(MainWindow)
        self.actionStatus_Bar.setCheckable(True)
        self.actionStatus_Bar.setChecked(True)
        self.actionStatus_Bar.setObjectName("actionStatus_Bar")
        self.actionPrevious_Feed = QtGui.QAction(MainWindow)
        self.actionPrevious_Feed.setObjectName("actionPrevious_Feed")
        self.actionPrevious_Unread_Feed = QtGui.QAction(MainWindow)
        self.actionPrevious_Unread_Feed.setObjectName("actionPrevious_Unread_Feed")
        self.actionPrevious_Article = QtGui.QAction(MainWindow)
        self.actionPrevious_Article.setObjectName("actionPrevious_Article")
        self.actionPrevious_Unread_Article = QtGui.QAction(MainWindow)
        self.actionPrevious_Unread_Article.setObjectName("actionPrevious_Unread_Article")
        self.actionFind = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/find.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionFind.setIcon(icon)
        self.actionFind.setObjectName("actionFind")
        self.actionFind_Again = QtGui.QAction(MainWindow)
        self.actionFind_Again.setObjectName("actionFind_Again")
        self.actionShow_Only_Unread_Feeds = QtGui.QAction(MainWindow)
        self.actionShow_Only_Unread_Feeds.setCheckable(True)
        self.actionShow_Only_Unread_Feeds.setObjectName("actionShow_Only_Unread_Feeds")
        self.actionOpen_Homepage = QtGui.QAction(MainWindow)
        self.actionOpen_Homepage.setObjectName("actionOpen_Homepage")
        self.actionDelete_Feed = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/editdelete.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionDelete_Feed.setIcon(icon)
        self.actionDelete_Feed.setObjectName("actionDelete_Feed")
        self.actionAdd_Feed = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/filenew.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionAdd_Feed.setIcon(icon)
        self.actionAdd_Feed.setObjectName("actionAdd_Feed")
        self.actionNew_Folder = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/folder_new.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionNew_Folder.setIcon(icon)
        self.actionNew_Folder.setObjectName("actionNew_Folder")
        self.actionEdit_Feed = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/edit.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionEdit_Feed.setIcon(icon)
        self.actionEdit_Feed.setObjectName("actionEdit_Feed")
        self.actionMark_as_Read = QtGui.QAction(MainWindow)
        self.actionMark_as_Read.setObjectName("actionMark_as_Read")
        self.actionMark_as_Unread = QtGui.QAction(MainWindow)
        self.actionMark_as_Unread.setObjectName("actionMark_as_Unread")
        self.actionMark_as_Important = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/flag.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionMark_as_Important.setIcon(icon)
        self.actionMark_as_Important.setObjectName("actionMark_as_Important")
        self.actionDelete_Article = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/editdelete.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionDelete_Article.setIcon(icon)
        self.actionDelete_Article.setObjectName("actionDelete_Article")
        self.actionOpen_in_Browser = QtGui.QAction(MainWindow)
        self.actionOpen_in_Browser.setObjectName("actionOpen_in_Browser")
        self.actionRemove_Important_Mark = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/flag.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionRemove_Important_Mark.setIcon(icon)
        self.actionRemove_Important_Mark.setObjectName("actionRemove_Important_Mark")
        self.actionExpire_Feed = QtGui.QAction(MainWindow)
        self.actionExpire_Feed.setObjectName("actionExpire_Feed")
        self.actionTechnorati_Top_100 = QtGui.QAction(MainWindow)
        self.actionTechnorati_Top_100.setObjectName("actionTechnorati_Top_100")
        self.actionNormal_View = QtGui.QAction(MainWindow)
        self.actionNormal_View.setEnabled(False)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/view_top_bottom.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionNormal_View.setIcon(icon)
        self.actionNormal_View.setObjectName("actionNormal_View")
        self.actionCombined_View = QtGui.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/view_detailed.svg"),QtGui.QIcon.Normal,QtGui.QIcon.Off)
        self.actionCombined_View.setIcon(icon)
        self.actionCombined_View.setObjectName("actionCombined_View")
        self.toolBar.addAction(self.actionFetch_Feed)
        self.toolBar.addAction(self.actionFetch_All_Feeds)
        self.toolBar.addAction(self.actionAbort_Fetches)
        self.toolBar.addAction(self.actionMark_Feed_as_Read)
        self.menuFeed.addSeparator()
        self.menuFeed.addAction(self.actionAdd_Feed)
        self.menuFeed.addAction(self.actionNew_Folder)
        self.menuFeed.addSeparator()
        self.menuFeed.addAction(self.actionMark_Feed_as_Read)
        self.menuFeed.addSeparator()
        self.menuFeed.addAction(self.actionFetch_Feed)
        self.menuFeed.addAction(self.actionFetch_All_Feeds)
        self.menuFeed.addAction(self.actionAbort_Fetches)
        self.menuFeed.addSeparator()
        self.menuFeed.addAction(self.actionEdit_Feed)
        self.menuFeed.addAction(self.actionExpire_Feed)
        self.menuFeed.addAction(self.actionDelete_Feed)
        self.menu_File.addAction(self.actionImport_Feeds)
        self.menu_File.addAction(self.actionExport_Feeds)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionTechnorati_Top_100)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionQuit)
        self.menu_Go.addSeparator()
        self.menu_Go.addAction(self.actionPrevious_Article)
        self.menu_Go.addAction(self.actionPrevious_Unread_Article)
        self.menu_Go.addAction(self.actionNext_Article)
        self.menu_Go.addAction(self.actionNext_Unread_Article)
        self.menu_Go.addSeparator()
        self.menu_Go.addAction(self.actionPrevious_Feed)
        self.menu_Go.addAction(self.actionPrevious_Unread_Feed)
        self.menu_Go.addAction(self.actionNext_Feed)
        self.menu_Go.addAction(self.actionNext_Unread_Feed)
        self.menuHelp.addAction(self.actionAbout_uRSSus)
        self.menu_View_Mode.addAction(self.actionNormal_View)
        self.menu_View_Mode.addAction(self.actionCombined_View)
        self.menu_View.addAction(self.actionStatus_Bar)
        self.menu_View.addSeparator()
        self.menu_View.addAction(self.actionShow_Only_Unread_Feeds)
        self.menu_View.addSeparator()
        self.menu_View.addAction(self.actionIncrease_Font_Sizes)
        self.menu_View.addAction(self.actionDecrease_Font_Sizes)
        self.menu_View.addSeparator()
        self.menu_View.addAction(self.menu_View_Mode.menuAction())
        self.menu_Edit.addAction(self.actionFind)
        self.menu_Edit.addAction(self.actionFind_Again)
        self.menuArticle.addAction(self.actionOpen_in_Browser)
        self.menuArticle.addSeparator()
        self.menuArticle.addAction(self.actionMark_as_Important)
        self.menuArticle.addAction(self.actionRemove_Important_Mark)
        self.menuArticle.addAction(self.actionMark_as_Read)
        self.menuArticle.addAction(self.actionMark_as_Unread)
        self.menuArticle.addAction(self.actionDelete_Article)
        self.menuBar.addAction(self.menu_File.menuAction())
        self.menuBar.addAction(self.menu_Edit.menuAction())
        self.menuBar.addAction(self.menu_View.menuAction())
        self.menuBar.addAction(self.menu_Go.menuAction())
        self.menuBar.addAction(self.menuFeed.menuAction())
        self.menuBar.addAction(self.menuArticle.menuAction())
        self.menuBar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "uRSSus", None, QtGui.QApplication.UnicodeUTF8))
        self.toolBar.setWindowTitle(QtGui.QApplication.translate("MainWindow", "toolBar", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFeed.setTitle(QtGui.QApplication.translate("MainWindow", "Fee&d", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_File.setTitle(QtGui.QApplication.translate("MainWindow", "&File", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_Go.setTitle(QtGui.QApplication.translate("MainWindow", "&Go", None, QtGui.QApplication.UnicodeUTF8))
        self.menuHelp.setTitle(QtGui.QApplication.translate("MainWindow", "&Help", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_View.setTitle(QtGui.QApplication.translate("MainWindow", "&View", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_View_Mode.setTitle(QtGui.QApplication.translate("MainWindow", "&View Mode", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_Edit.setTitle(QtGui.QApplication.translate("MainWindow", "&Edit", None, QtGui.QApplication.UnicodeUTF8))
        self.menuArticle.setTitle(QtGui.QApplication.translate("MainWindow", "&Article", None, QtGui.QApplication.UnicodeUTF8))
        self.filterBar.setWindowTitle(QtGui.QApplication.translate("MainWindow", "toolBar_2", None, QtGui.QApplication.UnicodeUTF8))
        self.actionFetch_Feed.setText(QtGui.QApplication.translate("MainWindow", "Fetch Feed", None, QtGui.QApplication.UnicodeUTF8))
        self.actionFetch_Feed.setShortcut(QtGui.QApplication.translate("MainWindow", "F5", None, QtGui.QApplication.UnicodeUTF8))
        self.actionFetch_All_Feeds.setText(QtGui.QApplication.translate("MainWindow", "Fetch All Feeds", None, QtGui.QApplication.UnicodeUTF8))
        self.actionFetch_All_Feeds.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+L", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAbort_Fetches.setText(QtGui.QApplication.translate("MainWindow", "Abort Fetches", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAbort_Fetches.setShortcut(QtGui.QApplication.translate("MainWindow", "Esc", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMark_Feed_as_Read.setText(QtGui.QApplication.translate("MainWindow", "Mark Feed as Read", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMark_Feed_as_Read.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+R", None, QtGui.QApplication.UnicodeUTF8))
        self.actionImport_Feeds.setText(QtGui.QApplication.translate("MainWindow", "&Import Feeds...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionExport_Feeds.setText(QtGui.QApplication.translate("MainWindow", "&Export Feeds...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionQuit.setText(QtGui.QApplication.translate("MainWindow", "&Quit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionQuit.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+Q", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNext_Article.setText(QtGui.QApplication.translate("MainWindow", "&Next Article", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNext_Article.setShortcut(QtGui.QApplication.translate("MainWindow", "Right", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNext_Unread_Article.setText(QtGui.QApplication.translate("MainWindow", "Ne&xt Unread Article", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNext_Unread_Article.setShortcut(QtGui.QApplication.translate("MainWindow", "+", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNext_Feed.setText(QtGui.QApplication.translate("MainWindow", "Next &Feed", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNext_Feed.setShortcut(QtGui.QApplication.translate("MainWindow", "N", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNext_Unread_Feed.setText(QtGui.QApplication.translate("MainWindow", "N&ext Unread Feed", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNext_Unread_Feed.setShortcut(QtGui.QApplication.translate("MainWindow", "Alt++", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAbout_uRSSus.setText(QtGui.QApplication.translate("MainWindow", "&About uRSSus...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionIncrease_Font_Sizes.setText(QtGui.QApplication.translate("MainWindow", "Increase Font Sizes", None, QtGui.QApplication.UnicodeUTF8))
        self.actionIncrease_Font_Sizes.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl++", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDecrease_Font_Sizes.setText(QtGui.QApplication.translate("MainWindow", "Decrease Font Sizes", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDecrease_Font_Sizes.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+-", None, QtGui.QApplication.UnicodeUTF8))
        self.actionStatus_Bar.setText(QtGui.QApplication.translate("MainWindow", "Status Bar", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPrevious_Feed.setText(QtGui.QApplication.translate("MainWindow", "P&revious Feed", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPrevious_Feed.setShortcut(QtGui.QApplication.translate("MainWindow", "P", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPrevious_Unread_Feed.setText(QtGui.QApplication.translate("MainWindow", "Prev&ious Unread Feed", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPrevious_Unread_Feed.setShortcut(QtGui.QApplication.translate("MainWindow", "Alt+-", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPrevious_Article.setText(QtGui.QApplication.translate("MainWindow", "&Previous Article", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPrevious_Article.setShortcut(QtGui.QApplication.translate("MainWindow", "Left", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPrevious_Unread_Article.setText(QtGui.QApplication.translate("MainWindow", "Pre&vious Unread Article", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPrevious_Unread_Article.setShortcut(QtGui.QApplication.translate("MainWindow", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.actionFind.setText(QtGui.QApplication.translate("MainWindow", "&Find", None, QtGui.QApplication.UnicodeUTF8))
        self.actionFind.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+F", None, QtGui.QApplication.UnicodeUTF8))
        self.actionFind_Again.setText(QtGui.QApplication.translate("MainWindow", "Find Again", None, QtGui.QApplication.UnicodeUTF8))
        self.actionFind_Again.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+G", None, QtGui.QApplication.UnicodeUTF8))
        self.actionShow_Only_Unread_Feeds.setText(QtGui.QApplication.translate("MainWindow", "Show Only Unread Feeds", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen_Homepage.setText(QtGui.QApplication.translate("MainWindow", "Open Homepage", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen_Homepage.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+H", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDelete_Feed.setText(QtGui.QApplication.translate("MainWindow", "&Delete Feed", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDelete_Feed.setShortcut(QtGui.QApplication.translate("MainWindow", "Alt+Del", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAdd_Feed.setText(QtGui.QApplication.translate("MainWindow", "Add Feed", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAdd_Feed.setShortcut(QtGui.QApplication.translate("MainWindow", "Ins", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_Folder.setText(QtGui.QApplication.translate("MainWindow", "New Folder", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_Folder.setShortcut(QtGui.QApplication.translate("MainWindow", "Shift+Ins", None, QtGui.QApplication.UnicodeUTF8))
        self.actionEdit_Feed.setText(QtGui.QApplication.translate("MainWindow", "&Edit Feed", None, QtGui.QApplication.UnicodeUTF8))
        self.actionEdit_Feed.setShortcut(QtGui.QApplication.translate("MainWindow", "F2", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMark_as_Read.setText(QtGui.QApplication.translate("MainWindow", "Mark as Read", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMark_as_Read.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+E", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMark_as_Unread.setText(QtGui.QApplication.translate("MainWindow", "Mark as Unread", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMark_as_Unread.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+U", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMark_as_Important.setText(QtGui.QApplication.translate("MainWindow", "Mark as Important", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMark_as_Important.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+I", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDelete_Article.setText(QtGui.QApplication.translate("MainWindow", "Delete Article", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDelete_Article.setShortcut(QtGui.QApplication.translate("MainWindow", "Del", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen_in_Browser.setText(QtGui.QApplication.translate("MainWindow", "Open in Browser", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen_in_Browser.setShortcut(QtGui.QApplication.translate("MainWindow", "Shift+Return", None, QtGui.QApplication.UnicodeUTF8))
        self.actionRemove_Important_Mark.setText(QtGui.QApplication.translate("MainWindow", "Remove Important Mark", None, QtGui.QApplication.UnicodeUTF8))
        self.actionRemove_Important_Mark.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+Shift+I", None, QtGui.QApplication.UnicodeUTF8))
        self.actionExpire_Feed.setText(QtGui.QApplication.translate("MainWindow", "Expire Feed", None, QtGui.QApplication.UnicodeUTF8))
        self.actionTechnorati_Top_100.setText(QtGui.QApplication.translate("MainWindow", "Import Technorati Top 100", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNormal_View.setText(QtGui.QApplication.translate("MainWindow", "Normal View", None, QtGui.QApplication.UnicodeUTF8))
        self.actionCombined_View.setText(QtGui.QApplication.translate("MainWindow", "Combined View", None, QtGui.QApplication.UnicodeUTF8))

from PyQt4 import QtWebKit
import icons_rc



# -*- coding: utf-8 -*-

# uRSSus, a multiplatform GUI news agregator
# Copyright (C) 2008 Roberto Alsina
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Singleton config object

import ConfigParser
import os
from demjson import JSON

cfdir=os.path.join(os.path.expanduser('~'),'.urssus')
cfname=os.path.join(cfdir,'config')

def getValue(section,key,default=None):
  section=section.lower()
  key=key.lower()
  try:
    return JSON().decode(conf.get (section,key))
  except:
    return default

def setValue(section,key,value):
  section=str(section)
  key=str(key)
  section=section.lower()
  key=key.lower()
  value=JSON().encode(value)
  try:
    r=conf.set(section,key,value)
  except ConfigParser.NoSectionError:
    conf.add_section(section)
    r=conf.set(section,key,value)
  f=open(cfname,'w')
  conf.write(f)
  return r


class ConfigError(Exception):
  def __init__(self,modulename,msg):
    self.modulename=modulename
    self.msg=msg


conf=ConfigParser.SafeConfigParser()
if not os.path.isdir(cfdir):
  os.mkdir(cfdir)

if not os.path.isfile(cfname):
  open(cfname, 'w').close()
f=open(cfname,'r')
conf.readfp(f)
f.close()



